package com.fsense;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.util.Properties;
import java.util.Set;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import com.fsense.service.IService;
import com.fsense.service.impl.Service;
import com.fsense.utils.StringUtils;

public class App {
	static Logger logger = Logger.getLogger(App.class);

	public static void main(String[] args) {
		try {
			if (args == null || args.length < 1) {
				// ERROR
				logger.debug("APPLICATION FAILED");
				logger.debug("======================================================");
				logger.error("Configuration file not informed to start execution.");
				logger.debug("======================================================");
				System.exit(-1);
			}
			// INFO
			logger.debug("APPLICATION STARTED");
			logger.debug("======================================================");
			logger.info("Config file found in path: \n [" + args[0] + "]");
			logger.debug("======================================================");
			Properties context = getContext(args);

			IService service = getServiceInstance(context);
			service.execute(context);
		} catch (Exception e) {
			logger.error(e.getMessage());
		} finally {
			logger.info("End of execution.");
			System.exit(1);
		}
	}

	private static Properties getContext(String... args) throws IOException {
		if (args.length == 0 || StringUtils.isEmpty(args[0])) {
			logger.debug("======================================================");
			logger.error("Please inform a configuration file required to start application. Example: fsense.properties");
			logger.debug("======================================================");
			System.exit(1);
		}
		Properties context = new Properties();
		context.load(Files.newInputStream(new File(args[0]).toPath()));
		return context;
	}

	private static IService getServiceInstance(Properties context) throws Exception {

		String serviceName = (String) context.get("service.name");

		logger.info("SELECTED SERVICE [" + serviceName + "]");
		logger.debug("======================================================");
		Service service = Service.getServiceByName(serviceName , context);
		
		if (service == null) {
			logger.error("No service found for informed name: " + serviceName);
			Set<String> services = Service.list(); 
			logger.info("Available services:");
			logger.debug("------------------------------------------------------");
			for(String svc : services) {
				logger.info("["+ svc +"]");
			}
			logger.debug("======================================================");

			System.exit(1);
		}
		return service.service().getConstructor(Properties.class).newInstance(context);
	}
}
