package com.fsense.api;

import java.net.URI;

import org.apache.http.HttpHeaders;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.Logger;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fsense.entities.ActivityQuery;
import com.fsense.entities.ActivityResponse;
import com.fsense.entities.ListError;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.TokenResponse;
import com.fsense.exceptions.RequestException;

/***
 * classe responsavel por fazer chamadas http de atividades do usuario
 * 
 * @author 44944_wester
 *
 */
public class FSenseActivityRequest extends GenericRequest {
	private static final Logger logger = Logger.getLogger(FSenseActivityRequest.class);

	public FSenseActivityRequest(ClientRequestIntegrationConfiguration configuration) {
		super(configuration);
	}

	/***
	 * Este método faz uma requisição http GET no endpoint /activity e buscar
	 * atividades dentro do periodo informado como parâmetro
	 * 
	 * @param token
	 * @param activityQuery
	 * @return
	 * @throws Exception
	 */
	public ActivityResponse getActivities(TokenResponse token, ActivityQuery activityQuery) throws Exception {
		CloseableHttpClient httpClient = HttpClients.createDefault();

		URIBuilder uriBuilder = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/activity");
		
		//prepara a requisição http com parametros obrigatorios
		uriBuilder.setParameter("startDate", activityQuery.getStartDate());
		uriBuilder.setParameter("endDate", activityQuery.getEndDate());
		uriBuilder.setParameter("page", String.valueOf(activityQuery.getPage()));
		
		//prepara a requisição com parametros opcionais
		if (activityQuery.getLimit() != null) {
			uriBuilder.setParameter("limit", String.valueOf(activityQuery.getLimit()));
		}
		else {
			uriBuilder.setParameter("limit", String.valueOf(10));
		}
		if (activityQuery.getUser() != null) {
			uriBuilder.addParameter("user", activityQuery.getUser());
		}
		if (activityQuery.getApplication() != null) {
			uriBuilder.addParameter("application", activityQuery.getApplication());
		}

		URI uri = uriBuilder.build();

		HttpGet request = new HttpGet(uri);
		//add o token no header da requisição
		request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

		try (CloseableHttpResponse response = httpClient.execute(request)) {
			
			int statusCode = response.getStatusLine().getStatusCode();
			String responseBody = EntityUtils.toString(response.getEntity());
			ObjectMapper objectMapper = objectMapperFactory();
			
			//corpo da resposta do servidor em caso de sucesso
			ActivityResponse activityResponse;
			if (statusCode == 200) {
				//em caso de sucesso, retorna corpo da resposta para camada de serviço
				activityResponse = objectMapper.readValue(responseBody, ActivityResponse.class);
				return activityResponse;	
			} else {
				//em caso de erro, exibe a lista de erro gerado pelo servidor no console do sistema
				ListError listError = objectMapper.readValue(responseBody, ListError.class);
				exceptionUtils().printListErrors(listError);
				logger.error(listError);
				throw new RequestException("Error fetching activities from endpoint GET /activities");
			}
		} catch (RequestException e) {
			throw new Exception(e.getMessage());
		}
	}
}
