package com.fsense.api;

import java.net.URI;

import org.apache.http.HttpHeaders;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.Logger;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.EventQuery;
import com.fsense.entities.EventsResponse;
import com.fsense.entities.ListError;
import com.fsense.entities.TokenResponse;
import com.fsense.exceptions.RequestException;

/***
 * classe responsavel por fazer chamadas http de eventos de usuario
 * 
 * @author 44944_wester
 *
 */
public class FSenseEventsRequest extends GenericRequest {
	private static final Logger logger = Logger.getLogger(FSenseEventsRequest.class);

	public FSenseEventsRequest(ClientRequestIntegrationConfiguration configuration) {
		super(configuration);
	}

	/***
	 * Este método faz uma requisição http GET no endpoint /event e buscar eventos
	 * dentro do periodo informado como parâmetro
	 * 
	 * @param token
	 * @param eventQuery
	 * @return
	 * @throws Exception
	 */
	public EventsResponse getEvents(TokenResponse token, EventQuery eventQuery) throws Exception {
		CloseableHttpClient httpClient = HttpClients.createDefault();
		
			
			URIBuilder uriBuilder = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/event");
			
			//prepara a requisição com parametros obrigatorios
			uriBuilder.setParameter("page", String.valueOf(eventQuery.getPage()));
			uriBuilder.setParameter("login", eventQuery.getLogin());
			
			//prepara a requisiçao com parametros opcionais
			if (eventQuery.getStartDate() != null) {
				uriBuilder.setParameter("startDate", eventQuery.getStartDate());
			}
			else {
				uriBuilder.setParameter("startCreationDate", eventQuery.getStartCreationDate());
			}
			if (eventQuery.getEndDate() != null) {
				uriBuilder.setParameter("endDate", eventQuery.getEndDate());
			}
			else {
				uriBuilder.setParameter("endCreationDate", eventQuery.getEndCreationDate());
			}
			if (eventQuery.getApplication() != null) {
				uriBuilder.setParameter("application", eventQuery.getApplication());
			}
			if (eventQuery.getCategory() != null) {
				uriBuilder.setParameter("category", eventQuery.getCategory());
			}
			if (eventQuery.getLimit() != null) {
				uriBuilder.setParameter("limit", String.valueOf(eventQuery.getLimit()));
			}
			else {
				uriBuilder.setParameter("limit", String.valueOf(10));
			}
			

			URI uri = uriBuilder.build();

			HttpGet request = new HttpGet(uri);
			
			//add o token no header da requisição
			request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

			try (CloseableHttpResponse response = httpClient.execute(request)) {
				int statusCode = response.getStatusLine().getStatusCode();
				String responseBody = EntityUtils.toString(response.getEntity());
				ObjectMapper objectMapper = objectMapperFactory();
				
				//objeto compativel com o corpo da resposta do servidor
				EventsResponse eventsResponse;
				if (statusCode == 200) {
					//em caso de  sucesso, retorna o objeto da resposta para a camada de serviço
					eventsResponse = objectMapper.readValue(responseBody, EventsResponse.class);
					return eventsResponse;
				} else {
					// em caso de erro, exibe a lista de erro gerado pelo servidor no console do
					// sistema
					ListError listError = objectMapper.readValue(responseBody, ListError.class);
					exceptionUtils().printListErrors(listError);
					logger.error(listError);
					throw new RequestException("Error fetching events from endpoint GET /events");
				}
			} catch (RequestException e) {
				throw new Exception(e.getMessage());
			}
	}
}
