package com.fsense.api;

import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;

import org.apache.http.HttpHeaders;
import org.apache.http.client.utils.URIBuilder;
import org.apache.log4j.Logger;

import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.KeystrokesQuery;
import com.fsense.entities.KeystrokesResponse;
import com.fsense.entities.ListError;
import com.fsense.entities.TokenResponse;

public class FSenseKeystrokesRequest extends GenericRequest {
	private static final Logger logger = Logger.getLogger(FSenseKeystrokesRequest.class);
	
	public FSenseKeystrokesRequest(ClientRequestIntegrationConfiguration configuration) {
		super(configuration);
	}
	
	public KeystrokesResponse getKeystrokes(TokenResponse token, KeystrokesQuery keystrokesQuery) throws Exception {
		URIBuilder uriBuilder = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/keystrokes")
				.addParameter("page", String.valueOf(keystrokesQuery.getPage()))
				.addParameter("limit", String.valueOf(keystrokesQuery.getLimit()))
				.addParameter("startDateTime", String.valueOf(keystrokesQuery.getStartDateTime()))
				.addParameter("endDateTime", String.valueOf(keystrokesQuery.getEndDateTime()));
		
		if(!keystrokesQuery.getUserNames().isEmpty()) {
			String usernames = String.join(",", keystrokesQuery.getUserNames());
			uriBuilder.addParameter("userNames", String.valueOf(usernames));
		}
		
		HttpClient httpClient = HttpClient.newHttpClient();
		
		HttpRequest request = HttpRequest.newBuilder().uri(uriBuilder.build()).GET()
				.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken()).build();

		HttpResponse<String> response = httpClient.send(request, HttpResponse.BodyHandlers.ofString());
		if (response.statusCode() == 200) {
			KeystrokesResponse keystrokesResponse = objectMapperFactory().readValue(response.body(),
					KeystrokesResponse.class);
			return keystrokesResponse;
		} else {
			ListError listError = objectMapperFactory().readValue(response.body(), ListError.class);
			logger.error(listError);
			throw new Exception("Error fetching keystrokes from endpont GET /keystrokes");
		}
	}

}
