package com.fsense.api;

import java.net.URI;

import org.apache.http.HttpHeaders;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.Logger;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fsense.entities.ListError;
import com.fsense.entities.LoginHistoryQuery;
import com.fsense.entities.LoginHistoryResponse;
import com.fsense.entities.ClientRequestIntegrationConfiguration;
import com.fsense.entities.TokenResponse;
import com.fsense.exceptions.RequestException;

/****
 *  classe responsavel por fazer chamadas http para operações no endpoint de
 *  historico de Login
 *
 * @author 44944_wester
 *
 */
public class FSenseLoginHistoryRequest extends GenericRequest {
    private static final Logger logger = Logger.getLogger(FSenseLoginHistoryRequest.class);

    public FSenseLoginHistoryRequest(ClientRequestIntegrationConfiguration configuration) {
        super(configuration);
    }


    /***
     * este método faz uma chamada http GET no endpoint /login-history
     * busca lista de historico de login da organizaçao baseado nos parametros passados
     * @param token
     * @return
     * @throws Exception
     */
    public LoginHistoryResponse getLoginHistory(TokenResponse token, int page, LoginHistoryQuery loginHistoryQuery) throws Exception {
        CloseableHttpClient httpClient = HttpClients.createDefault();

        	//prepara o endpoint pra realizar a requisiçao
            URIBuilder uriBuilder = new URIBuilder(getBaseUrl() + "/" + getApiVersion() + "/login-history");
            
            //prepara os parametros opcionais da requisiçao
            if (loginHistoryQuery.getPersonId() != null) {
                uriBuilder.setParameter("personId", String.valueOf(loginHistoryQuery.getPersonId()));
            }
            if (loginHistoryQuery.getStart() != null) {
            	uriBuilder.setParameter("start", loginHistoryQuery.getStart());
            }
            if (loginHistoryQuery.getEnd() != null) {
            	uriBuilder.setParameter("end", loginHistoryQuery.getEnd());
            }
            
            URI uri = uriBuilder.build();

            HttpGet request = new HttpGet(uri);
            
            //add o token no header da requisição
            request.setHeader(HttpHeaders.AUTHORIZATION, "Bearer " + token.getAccessToken());

            try (CloseableHttpResponse response = httpClient.execute(request)) {
                String responseBody = EntityUtils.toString(response.getEntity());
                int statusCode = response.getStatusLine().getStatusCode();
                ObjectMapper objectMapper = objectMapperFactory();
                
                //objeto compativel com o corpo da resposta do server
                LoginHistoryResponse loginHistoryResponse;
                if (statusCode == 200) {
                	//em caso de sucesso, retorna o objeto armazenando a resposta para a camada de servico
                    loginHistoryResponse = objectMapper.readValue(responseBody, LoginHistoryResponse.class);
                    return loginHistoryResponse;
                } else {
                	// em caso de erro, exibe a lista de erro gerado pelo servidor no console do
    				// sistema
                    ListError listError = objectMapper.readValue(responseBody, ListError.class);
                    exceptionUtils().printListErrors(listError);
                    logger.error(listError);
                    throw new RequestException("An error occurred when fetch login history from endpoint GET /login-history");
                }
        } catch (RequestException e) {
            throw new Exception(e.getMessage());
        }
    }
}
