package com.fsense.authentication;

import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.Logger;

import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fsense.entities.AuthError;
import com.fsense.entities.TokenRequest;
import com.fsense.entities.TokenResponse;
import com.fsense.exceptions.ExceptionUtils;

//classe responsavel por fazer a chamada http pra autenticação do usuario
public class AccessTokenRequest {

    static Logger logger = Logger.getLogger(AccessTokenRequest.class);

    TokenRequest tokenRequest = new TokenRequest();
    //Este método processa a requisição pra buscar um token de acesso ao demais serviços do sistema
    public TokenResponse getAccessToken(Properties context) throws Exception {
        logger.debug("========================================================");
        logger.debug("Starting authentication process");
        logger.debug("Getting access token");
        
        tokenRequest.setClientId(context.getProperty("client.id"));
        tokenRequest.setClientSecret(context.getProperty("client.secret"));

        String baseUrl = context.getProperty("api.base.url");
        
        if(baseUrl.isBlank()) {
        	throw new Exception("BaseURL not informed, please set property api.base.url in file fsense.properties");
        }

        //Cria uma conexao Http
        CloseableHttpClient httpClient = HttpClients.createDefault();
        
        //Url para autenticação e obtenção do token de acesso
        HttpPost httpPost = new HttpPost(baseUrl + "/oauth/token");

        //Passagem de parâmetros necessários pra autenticação
        List<NameValuePair> params = new ArrayList<>();
        params.add(new BasicNameValuePair("grant_type", tokenRequest.getGrantType()));
        params.add(new BasicNameValuePair("client_id", tokenRequest.getClientId()));
        if(tokenRequest.getClientId().isBlank()) {
        	throw new Exception("client id not informed, please set property client.id in file fsense.properties");
        }
        params.add(new BasicNameValuePair("client_secret", tokenRequest.getClientSecret()));
        if(tokenRequest.getClientSecret().isBlank()) {
        	throw new Exception("client secret not informed, please set property client.secret in file fsense.properties");
        }
        httpPost.setEntity(new UrlEncodedFormEntity(params));

        //Se sucesso, resposta é processada,
        // e o token é armazenado num objeto TokenResponse compativel com a resposta do servidor
        try (CloseableHttpResponse response = httpClient.execute(httpPost)) {
        	int statusCode = response.getStatusLine().getStatusCode();
            HttpEntity entity = response.getEntity();
            String responseBody = EntityUtils.toString(entity);
            ObjectMapper objectMapper = new ObjectMapper();
            if(statusCode == 200) {
            	TokenResponse tokenResponse = objectMapper.readValue(responseBody, TokenResponse.class);
                logger.debug("Token received.");
                logger.debug(tokenResponse.getAccessToken());
                logger.debug("========================================================");
                logger.debug("AUTHENTICATION SUCCESS");
                logger.debug("========================================================");
                return tokenResponse;
            }
            else {
            	ObjectMapper om = new ObjectMapper();
            	AuthError error = om.readValue(responseBody, AuthError.class);
            	logger.error("error: " + error.getError());
            	logger.error("error description: " + error.getErrorDescription());
            	logger.debug("========================================================");
                logger.debug("AUTHENTICATION FAILED");
                logger.debug("========================================================");
                throw new Exception("Invalid credentials");
            }    
        }
        //lançado erro na requisição do token, possíveis causas credenciais invalidas
        //ou inseridas incorretamente.
        catch (Exception e) {
        	throw new Exception(e.getMessage());
        }
    }
    
    public ExceptionUtils exceptionUtils() {
		return new ExceptionUtils();
	}
    
    protected static ObjectMapper objectMapperFactory() {
		ObjectMapper objectMapper = new ObjectMapper();
		objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
		return objectMapper;
	}
}
