package com.fsense.jdbc;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.exceptions.DatabaseException;
/**
 * Classe responsavel por criar e fechar conexoes de banco de dados, transações
 * @author 44944_wester
 *
 */
public class DB {
	private static final Logger logger = Logger.getLogger(DB.class);
	
	private static Connection conn = null;
	
	//metodo abre uma conexao do banco de dados
	public static Connection getConnection(Properties context) {
		if (conn == null) {
			try {
				String StringConnection = context.getProperty("string.connection");
				if(StringConnection.isBlank()) {
					logger.error("String connection is Empty, please set property string.connection in file fsense.properties and try again!");
				}
				String user = context.getProperty("database.user");
				if(user.isBlank()) {
					logger.error("Database User is Empty, please set property database.user in file fsense.properties and try again!");
				}
				String password = context.getProperty("database.password");
				if(password.isBlank()) {
					logger.error("Database Password is Empty, please set property database.password in file fsense.properties and try again!");
				}
				conn = DriverManager.getConnection(StringConnection, user, password);
			}
			catch (SQLException e) {
				throw new DatabaseException(e.getMessage());
			}
		}
		return conn;
	}
	
	//finaliza a conexao com banco de dados
	public static void closeConnection() {
		if(conn != null) {
			try {
				conn.close();
			}
			catch (SQLException e) {
				throw new DatabaseException(e.getMessage());
			}
		}
	}
	
	//finaliza a transação
	public static void closeStatement(Statement st) {
		if(st != null) {
			try {
				st.close();
			} catch (SQLException e) {
				throw new DatabaseException(e.getMessage());
			}
		}
	}
	
	//finaliza o resulSet
	public static void closeResultSet(ResultSet rs) {
		if(rs != null) {
			try {
				rs.close();
			}
			catch (SQLException e) {
				throw new DatabaseException(e.getMessage());
			}
		}
	}
}
