package com.fsense.jdbc.impl;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Timestamp;

import org.apache.log4j.Logger;

import com.fsense.entities.Event;
import com.fsense.jdbc.DB;
import com.fsense.jdbc.dao.EventDAO;

/***
 * Classe que implementa as operações da implementação de banco de dados do serviço de Eventos
 * @author 44944_wester
 *
 */
public class EventDAOImpl implements EventDAO {
	private static final Logger logger = Logger.getLogger(EventDAOImpl.class);

	private Connection conn;

	public EventDAOImpl(Connection conn) {
		this.conn = conn;
	}

	@Override
	public void insert(Event evt) {
		PreparedStatement st = null;
		try {
			st = conn.prepareStatement("INSERT INTO tb_event (application, login, process_name, "
					+ "evt_duration_milliseconds, host, url, machine, internal_ip, evt_fired, "
					+ "evt_stopped, creation_date, mouse_clicks_per_millisecond, keystrokes_per_millisecond, mouse_clicks_count, keystrokes_count) " 
					+ "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");

			st.setString(1, evt.getApplication());
			st.setString(2, evt.getLogin());
			st.setString(3, evt.getProcessName());
			st.setInt(4, evt.getEvtDurationMilliseconds());
			st.setString(5, evt.getHost());
			st.setString(6, evt.getUrl());
			st.setString(7, evt.getMachine());
			st.setString(8, evt.getInternalIp());
			st.setTimestamp(9, Timestamp.valueOf(evt.getEvtFired().replace("T", " ").replace("+00:00", "")));
			st.setTimestamp(10, Timestamp.valueOf(evt.getEvtStopped().replace("T", " ").replace("+00:00", "")));
			st.setTimestamp(11, Timestamp.valueOf(evt.getCreationDate().replace("T", " ").replace("+00:00", "")));
			st.setDouble(12, (evt.getMouseClicksPerMillisecond() != null ? evt.getMouseClicksPerMillisecond() : 0.0));
			st.setDouble(13, (evt.getKeystrokesPerMillisecond() != null ? evt.getKeystrokesPerMillisecond() : 0.0));
			st.setInt(14, (evt.getMouseClicksCount() != null ? evt.getMouseClicksCount() : 0));
			st.setInt(15, (evt.getKeystrokesCount() != null ? evt.getKeystrokesCount() : 0));
			
			st.executeUpdate();
			
		} catch (SQLException e) {
			logger.error(e.getMessage());
		} finally {
			DB.closeStatement(st);
		}
	}
}
