package com.fsense.jdbc.impl;

import java.sql.*;

import org.apache.log4j.Logger;

import com.fsense.entities.Holiday;
import com.fsense.jdbc.DB;
import com.fsense.jdbc.dao.HolidayDAO;
/***
 * Classe que implementa as operações da implementação de banco de dados do serviço de Feriados
 * @author 44944_wester
 *
 */
public class HolidayServiceDAOImpl implements HolidayDAO {
	private static final Logger logger = Logger.getLogger(HolidayServiceDAOImpl.class);
	
	private Connection conn;
	
	public HolidayServiceDAOImpl(Connection conn) {
		this.conn = conn;
	}
	
	@Override
	public void insert(Holiday holiday) {
		PreparedStatement st = null;
		try {
			st = conn.prepareStatement("INSERT INTO tb_holiday "
					+ "(id, name, type, date, can_repeat, year, month, day, calendar_id) "
					+ "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)");
			
			st.setInt(1, holiday.getId());
			st.setString(2, holiday.getName());
			st.setString(3, holiday.getType().toString());
			st.setTimestamp(4, Timestamp.from(holiday.getDate().toInstant()));
			st.setBoolean(5, holiday.isCanRepeat());
			st.setInt(6, holiday.getYear());
			st.setInt(7, holiday.getMonth());
			st.setInt(8, holiday.getDay());
			if(holiday.getCalendarId() == null) {
				st.setNull(9, Types.INTEGER);
			}
			else {
				st.setInt(9, holiday.getCalendarId());
			}
			
			st.execute();
		}
		catch(SQLException e) {
			logger.error(e.getMessage());
		}
		finally {
			DB.closeStatement(st);
		}
	}
	
}
