package com.fsense.jdbc.impl;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.time.OffsetDateTime;
import java.time.format.DateTimeFormatter;

import org.apache.log4j.Logger;

import com.fsense.entities.Keystrokes;
import com.fsense.jdbc.DB;
import com.fsense.jdbc.dao.KeystrokesDAO;

public class KeystrokesDAOImpl implements KeystrokesDAO {
	private static final Logger logger = Logger.getLogger(KeystrokesDAOImpl.class);
	
	private Connection conn;
	
	public KeystrokesDAOImpl(Connection conn) {
		this.conn = conn;
	}
	
	@Override
	public void save(Keystrokes keystrokes) {
		PreparedStatement st = null;
		try {
			st = conn.prepareStatement("INSERT INTO tb_keystrokes "
					+ "(machine_name, user_name, creation_date, person_id, device_id, keystrokes_count, mouse_clicks_count, keystrokes_per_millisecond, mouse_clicks_per_millisecond, start_time, end_time, duration) "
					+ "VALUES(?,?,?,?,?,?,?,?,?,?,?,?)");
			st.setString(1, keystrokes.getMachineName());
			st.setString(2, keystrokes.getUserName());
			OffsetDateTime OffSetCreationDate = OffsetDateTime.parse(keystrokes.getCreationDate());
			String creationDateStr = OffSetCreationDate.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
			st.setTimestamp(3, Timestamp.valueOf(creationDateStr));
			st.setInt(4, keystrokes.getPersonId());
			st.setInt(5, keystrokes.getDeviceId());
			st.setInt(6, keystrokes.getKeystrokesCount());
			st.setInt(7, keystrokes.getMouseClicksCount());
			st.setDouble(8, keystrokes.getKeystrokesPerMillisecond());
			st.setDouble(9, keystrokes.getMouseClicksPerMillisecond());
			OffsetDateTime OffSetStart = OffsetDateTime.parse(keystrokes.getStart());
			String startStr = OffSetStart.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
			st.setTimestamp(10, Timestamp.valueOf(startStr));
			OffsetDateTime OffSetEnd = OffsetDateTime.parse(keystrokes.getCreationDate());
			String endStr = OffSetEnd.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss"));
			st.setTimestamp(11, Timestamp.valueOf(endStr));
			st.setInt(12, keystrokes.getDuration());
			
			st.executeUpdate();
		}
		catch (SQLException e) {
			logger.error(e.getMessage());
		}
		finally {
			DB.closeStatement(st);
		}
	}

}
