package com.fsense.jdbc.impl;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;

import org.apache.log4j.Logger;

import com.fsense.entities.Person;
import com.fsense.jdbc.DB;
import com.fsense.jdbc.dao.PersonDAO;
/***
 * Classe que implementa as operações da implementação de banco de dados do serviço de Pessoas
 * @author 44944_wester
 *
 */
public class PersonDAOImpl implements PersonDAO {
	private static final Logger logger = Logger.getLogger(PersonDAOImpl.class);
	
	private Connection conn;
	
	public PersonDAOImpl(Connection conn) {
		this.conn = conn;
	}
	
	@Override
	public void insert(Person person) {
		PreparedStatement st = null;
		try {
			st = conn.prepareStatement("INSERT INTO tb_person "
					+ "(id, name, email, device_identifier, team_id, team_name, workday_id, can_delete, root, calendar_id) "
					+ "VALUES "
					+ "(?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
			
			st.setInt(1, person.getId());
			st.setString(2, person.getName());
			if(person.getEmail() == null) {
				st.setNull(3, Types.VARCHAR);
			}else {
				st.setString(3, person.getEmail());
			}
			if(person.getDeviceIdentifier() == null) {
				st.setNull(4, Types.VARCHAR);
			}
			else {
				st.setString(4, person.getDeviceIdentifier());
			}
			if(person.getTeamId() == null) {
				st.setNull(5, Types.INTEGER);
			}else {
				st.setInt(5, person.getTeamId());
			}
			if(person.getTeamName() == null) {
				st.setNull(6, Types.VARCHAR);
			} else {
				st.setString(6, person.getTeamName());
			}
			if(person.getWorkdayId() == null) {
				st.setNull(7, Types.INTEGER);
			} else {
				st.setInt(7, person.getWorkdayId());
			}
			st.setBoolean(8, person.isCanDelete());
			st.setBoolean(9, person.isRoot());
			if(person.getCalendarId() == null) {
				st.setNull(10, Types.INTEGER);
			} else {
				st.setInt(10, person.getCalendarId());
			}
			
			st.executeUpdate();
		}
		catch (SQLException e) {
			logger.error(e.getMessage());
		}
		finally {
			DB.closeStatement(st);
		}
	}
}
