package com.fsense.jdbc.impl;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Types;

import org.apache.log4j.Logger;

import com.fsense.entities.Workday;
import com.fsense.jdbc.DB;
import com.fsense.jdbc.dao.WorkdayDAO;
/***
 * Classe que implementa as operações da implementação de banco de dados do serviço de Jornadas
 * @author 44944_wester
 *
 */
public class WorkdayDAOImpl implements WorkdayDAO {
	private static final Logger logger = Logger.getLogger(WorkdayDAOImpl.class);
	
	private Connection conn;
	
	public WorkdayDAOImpl(Connection conn) {
		this.conn = conn;
	}

	@Override
	public void insert(Workday workday) {
		PreparedStatement st = null;
		try {
			st = conn.prepareStatement("INSERT INTO tb_workday "
					+ "(id, name, work_national_holidays, work_regional_holidays, is_flexible, ignore_non_focus_longer_than, calendar_id)"
					+ "VALUES "
					+ "(?, ?, ?, ?, ?, ?, ?)");
			st.setInt(1, workday.getId());
			st.setString(2, workday.getName());
			st.setBoolean(3, workday.isWorkNationalHolidays());
			st.setBoolean(4, workday.isWorkRegionalHolidays());
			st.setBoolean(5, workday.isFlexible());
			if(workday.getIgnoreNonFocusLongerThan() == null) {
				st.setNull(6, Types.TIME);
			}
			else {
				st.setTime(6, Time.valueOf(workday.getIgnoreNonFocusLongerThan()));
			}
			if(workday.getCalendarId() == null) {
				st.setNull(7, Types.INTEGER);
			}
			else {
				st.setInt(7, workday.getCalendarId());
			}
			st.executeUpdate();
		}
		catch (SQLException e) {
			logger.error(e.getMessage());
		}
		finally {
			DB.closeStatement(st);
		}
	}

}
