package com.fsense.jdbc.impl;

import java.sql.Connection;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Types;

import org.apache.log4j.Logger;

import com.fsense.entities.WorkdayOverride;
import com.fsense.jdbc.DB;
import com.fsense.jdbc.dao.WorkdayOverrideDAO;
/***
 * Classe que implementa as operações da implementação de banco de dados do serviço de Trocas de Jornada
 * @author 44944_wester
 *
 */
public class WorkdayOverrideDAOImpl implements WorkdayOverrideDAO {
	private static final Logger logger = Logger.getLogger(WorkdayOverrideDAOImpl.class);
	
	private Connection conn;
	
	public WorkdayOverrideDAOImpl(Connection conn) {
		this.conn = conn;
	}

	@Override
	public void insert(WorkdayOverride workdayOverride) {
		PreparedStatement st = null;
		try {
			st = conn.prepareStatement("INSERT INTO tb_workday_override "
					+ "(id, person_id, date, start_time, interval_start_time, end_time, interval_end_time, creator_id, message, max_flexible_duration) "
					+ "VALUES "
					+ "(?, ?, ?, ?, ?, ?, ?, ? ,? ,?)");
			st.setInt(1, workdayOverride.getId());
			st.setInt(2, workdayOverride.getPerson().getId());
			st.setDate(3, Date.valueOf(workdayOverride.getDate()));
			st.setTime(4, Time.valueOf(workdayOverride.getStartTime()));
			if(workdayOverride.getIntervalStartTime() == null) {
				st.setNull(5, Types.TIME);
			}else {
				 st.setTime(5, Time.valueOf(workdayOverride.getIntervalStartTime()));
			}
			st.setTime(6, Time.valueOf(workdayOverride.getEndTime()));
			if(workdayOverride.getIntervalEndTime() == null) {
				st.setNull(7, Types.TIME);
			}else {
				 st.setTime(7, Time.valueOf(workdayOverride.getIntervalEndTime()));
			}
			if(workdayOverride.getCreator() == null) {
				st.setNull(8, Types.INTEGER);
			}
			else {
				st.setInt(8, workdayOverride.getCreator().getId());
			}
			st.setString(9, workdayOverride.getMessage());
			if(workdayOverride.getMaxFlexibleDuration() == null) {
				st.setNull(10, Types.TIME);
			}
			else {
				st.setTime(10, Time.valueOf(workdayOverride.getMaxFlexibleDuration()));
			}		
			st.executeUpdate();
		}
		catch(SQLException e) {
			logger.error(e.getMessage());
		}
		finally {
			DB.closeStatement(st);
		}
	}

}
