package com.fsense.jdbc.impl;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Types;

import org.apache.log4j.Logger;

import com.fsense.entities.WorkdaySchedule;
import com.fsense.jdbc.DB;
import com.fsense.jdbc.dao.WorkdayScheduleDAO;
/***
 * Classe que auxiliar que faz o relacionamento da Jornada com os dias da Semana em tabela para implementaçao de Banco de Dados de Jornada
 * @author 44944_wester
 *
 */
public class WorkdayScheduleDAOImpl implements WorkdayScheduleDAO {
	private static final Logger logger = Logger.getLogger(WorkdayScheduleDAOImpl.class);
	
	private Connection conn;
	
	public WorkdayScheduleDAOImpl(Connection conn) {
		this.conn = conn;
	}
	@Override
	public void insert(WorkdaySchedule schedule, int workdayId) {
		PreparedStatement st = null;
		try {
			st = conn.prepareStatement("INSERT INTO tb_workday_schedule "
					+ "(weekday, start_time, end_time, max_flexible_duration, workday_id) "
					+ "VALUES "
					+ "(?, ?, ?, ?, ?)");
				st.setString(1, schedule.getWeekday());
				if(schedule.getStart() == null) {
					st.setNull(2, Types.TIME);
				}
				else {
					Time start = Time.valueOf(schedule.getStart());
					st.setTime(2, start);
				}
				if(schedule.getEnd() == null) {
					st.setNull(3, Types.TIME);
				}
				else {
					Time end = Time.valueOf(schedule.getEnd());
					st.setTime(3, end);
				}
				if(schedule.getMaxFlexibleDuration() == null) {
					st.setNull(4, Types.TIME);
				}
				else {
					Time maxFlexibleDuration = Time.valueOf(schedule.getMaxFlexibleDuration());
					st.setTime(4, maxFlexibleDuration);
				}
				st.setInt(5, workdayId);
				st.executeUpdate();	
		} catch(SQLException e) {
			logger.error(e.getMessage());
		} finally {
			DB.closeStatement(st);
		}
		
	}

}
