package com.fsense.service.impl;

import java.lang.reflect.InvocationTargetException;
import java.util.Date;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.authentication.AccessTokenRequest;
import com.fsense.entities.TokenResponse;
import com.fsense.service.IService;

/***
 * Este método de serviço responsavel por fazer a autenticação do usuario no
 * FSense e obter o token de acesso para as demais operações
 * 
 * @author 44944_wester
 *
 */
public class AuthService extends GenericService implements IService {
	private static final Logger logger = Logger.getLogger(AuthService.class);
	
	public AuthService(Properties context) {
		super(context);
	}

	private AccessTokenRequest accessTokenRequest = new AccessTokenRequest();
	private TokenResponse token;

	private TokenResponse authenticate(Properties context) throws Exception {
		return accessTokenRequest.getAccessToken(context);
	}

	/***
	 * este método executa o método selecionado no arquivo de configurações
	 */
	@Override
	public void execute(Properties context) throws Exception {
		String methodName = context.getProperty("service.method");
		logger.info("SELECTED METHOD [" + methodName + "]");
		try {
			if (methodName.isEmpty()) {
				logger.error("No method has been selected.");
				logger.error("Available methods: ");
				printAvaliableMethods();
			} else {
				String methodNameCamelCase = toCamelCase(methodName);
				invokeMethodByName(methodNameCamelCase, context);
			}
		} catch (InvocationTargetException e) {
			System.err.println("Error executing method : " + methodName);
			Throwable cause = e.getCause();
			logger.error(cause);
		} catch (Exception e) {
			logger.error("Error executing " + methodName);
			logger.error("No method found for informed name:  " + methodName);
			printAvaliableMethods();
		}
	}

	/***
	 * Verifica se o token é valido ou nao esta expirado
	 * 
	 * @return
	 */
	private boolean tokenIsValid() {
		if (this.token == null) {
			return false;
		} else {
			long expiresInMilliseconds = this.token.getExpiresIn() * 1000;
			Date expirationDate = new Date(System.currentTimeMillis() + expiresInMilliseconds);
			if (expirationDate.after(new Date())) {
				return true;
			} else {
				return false;
			}
		}
	}

	/***
	 * Este método chama a função que busca o token e disponibliza o token para as
	 * demais classes
	 * 
	 * @param context
	 * @throws Exception
	 */
	public void getAccessToken(Properties context) throws Exception {
		try {
			getToken(context);
		}
		catch (Exception e) {
			logger.error(e.getMessage());
		}
	}

	/***
	 * Este método aciona a função que faz a chamada de obter, se nao nao existir ,
	 * ou se o token tiver expirado
	 */
	 public TokenResponse getToken(Properties context) throws Exception {
		if (this.token == null || !tokenIsValid()) {
			logger.info("checking if token is valid?: " + tokenIsValid());
			this.token = authenticate(context);
		}
		return this.token;
	}
};
