package com.fsense.service.impl;

import java.lang.reflect.Method;
import java.text.SimpleDateFormat;
import java.util.Properties;

public abstract class GenericService {

	public static final SimpleDateFormat RUNTIME_DATE_FORMAT = new SimpleDateFormat("dd/MM/yyyy");

	@SuppressWarnings("unused")
	private final Properties context;

	public GenericService(Properties context) {
		this.context = context;
	}

	public void invokeMethodByName(String methodName, Properties context) throws Exception {
		try {
			Method method = getClass().getMethod(methodName, Properties.class);
			method.invoke(this, context);
		} catch (NoSuchMethodException e) {
			throw new IllegalArgumentException("Metodo invalido: " + methodName);
		}
	}

	public void printAvaliableMethods() {
		Method[] methods = getClass().getMethods();
		for (Method method : methods) {
			if (method.getDeclaringClass().equals(this.getClass()) && !method.getName().equals("execute")
					&& !method.getName().equals("mappingMethods") && !method.getName().equals("getToken")) {
				System.out.println("[" + toConstantCase(method.getName() + "]"));
			}
		}
	}

	public static String toCamelCase(String input) {
		String[] words = input.split("_");
		StringBuilder camelCaseBuilder = new StringBuilder(words[0].toLowerCase());
		for (int i = 1; i < words.length; i++) {
			String word = words[i];
			camelCaseBuilder.append(word.substring(0, 1).toUpperCase()).append(word.substring(1).toLowerCase());
		}
		return camelCaseBuilder.toString();
	}

	public static String toConstantCase(String input) {
		String converted = input.replaceAll("(.)(\\p{Upper})", "$1_$2");
		return converted.toUpperCase();
	}
}
