package com.fsense.service.impl.csv;

import java.io.IOException;
import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.entities.Activity;
import com.fsense.service.impl.writer.ActivityServiceWriter;
import com.fsense.service.impl.writer.AuthServiceWriter;
import com.fsense.utils.CSVUtil;
import com.fsense.utils.GenericCSVGenerator;
/***
 * Classe que implementa escrita em CSV para serviço de Atividades
 * @author 44944_wester
 *
 */
public class ActivitiyServiceCSVWriter extends AuthServiceWriter implements ActivityServiceWriter {
	private static final Logger logger = Logger.getLogger(ActivitiyServiceCSVWriter.class);
	
	public ActivitiyServiceCSVWriter(Properties context) {
		super(context);
	}
	
	/***
	 * Salva as atividades em um arquivo CSV separados por dia
	 * 
	 * Este método gera um arquivo CSV contendo as atividades separadas por dia.
	 * O arquivo é salvo no caminho configurado no arquivo fsense.properties
	 * src/main/resources/export
	 * 
	 * Se nao houver atividades para a data, nenhum arquivo é gerado.
	 */
	@Override
	public void saveActivities(Properties context, List<Activity> activities, String date) throws Exception {
		if (activities.size() > 0) {
			logger.debug("========================================================");
			logger.info("EXPORTING TO CSV");
			logger.debug("========================================================");
			logger.info("Generating file activities_" + date + ".csv");
			activities.forEach(act -> act.setDate(date));
			try {
				GenericCSVGenerator.generateListCSV(activities,
						CSVUtil.validPath(context.getProperty("path.save")) + "activities_" + date + ".csv",
						CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
			} catch (IOException e) {
				e.printStackTrace();
			}
			logger.info("File: activities_" + date + ".csv saved succesfully");
			logger.info("File saved in path: " + CSVUtil.validPath(context.getProperty("path.save")));
			logger.info("Export activities of day " + date + " completed successfully.");
			logger.debug("========================================================");
		} else {
			logger.debug("No data found for this date: " + date);
			logger.debug("========================================================");
			return;
		}
	}
}
