package com.fsense.service.impl.csv;

import java.util.List;
import java.util.Properties;
import java.util.stream.Collectors;

import org.apache.log4j.Logger;

import com.fsense.dto.CategoryTeamDTO;
import com.fsense.entities.Category;
import com.fsense.entities.CategoryTeam;
import com.fsense.service.impl.writer.AuthServiceWriter;
import com.fsense.service.impl.writer.CategoryServiceWriter;
import com.fsense.utils.CSVUtil;
import com.fsense.utils.GenericCSVGenerator;
/***
 * Classe que implementa escrita/leitura em CSV para serviço de Categorias
 * @author 44944_wester
 *
 */
public class CategoryServiceCSVWriter extends AuthServiceWriter implements CategoryServiceWriter {
	private static final Logger logger = Logger.getLogger(CategoryServiceCSVWriter.class);
	
	public CategoryServiceCSVWriter(Properties context) {
		super(context);
	}
	
	/***
	 * Salva as categorias em um arquivo CSV.
	 * 
	 * Este método gera um arquivo CSV contendo as categorias da lista.
	 * O arquivo é salvo no caminho configurado no arquivo fsense.properties
	 * src/main/resources/export
	 */
	@Override
	public void saveCategories(Properties context, List<Category> categories) throws Exception {
		for (Category cat : categories) {
			cat.setUsername(context.getProperty("category.username"));
		}
		logger.debug("========================================================");
		logger.info("EXPORTING TO CSV");
		logger.debug("========================================================");
		logger.info("Generating file categories.csv");
		GenericCSVGenerator.generateListCSV(categories,
				CSVUtil.validPath(context.getProperty("path.save")) + "categories.csv", CSVUtil.getDelimiter(context),
				CSVUtil.getQualifier(context));

		logger.info("File: categories.csv saved successfully");
		logger.info("File saved in path: " + CSVUtil.validPath(context.getProperty("path.save") + "categories.csv"));
		logger.debug("========================================================");
		logger.info("Export categories completed successfully.");
		logger.debug("========================================================");
	}
	
	/***
	 * Salva as categorias gerais em um arquivo CSV.
	 * 
	 * Este método gera um arquivo CSV contendo as categorias gerais da lista.
	 * O arquivo é salvo no caminho configurado no arquivo fsense.properties
	 * src/main/resources/export
	 */
	@Override
	public void saveGeneralCategories(Properties context, List<Category> categories) throws Exception {
		logger.debug("========================================================");
		logger.info("EXPORTING TO CSV");
		logger.debug("========================================================");
		logger.info("Generating file general_categories.csv");
		GenericCSVGenerator.generateListCSV(categories,
				CSVUtil.validPath(context.getProperty("path.save")) + "general_categories.csv",
				CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
		logger.info("File: general_categories.csv saved successfully");
		logger.info("File saved in path: " + CSVUtil.validPath(context.getProperty("path.save") + "categories.csv"));
		logger.debug("========================================================");
		logger.info("Export general categories  completed successfully.");
		logger.debug("========================================================");
	}
	
	/***
	 * Salva as categorias por equipe em um arquivo CSV.
	 * 
	 * Este método gera um arquivo CSV contendo as categorias por equipe da lista.
	 * O arquivo é salvo no caminho configurado no arquivo fsense.properties
	 * src/main/resources/export
	 */
	@Override
	public void saveCategoriesByTeam(Properties context, List<CategoryTeam> categories) throws Exception {
		List<CategoryTeamDTO> categoryTeamDTOs = categories.stream().map(cat -> new CategoryTeamDTO(cat))
				.collect(Collectors.toList());
		logger.debug("========================================================");
		logger.info("EXPORTING TO CSV");
		logger.debug("========================================================");
		if (context.getProperty("category.team.id").isEmpty()) {
			logger.info("Generating file categories_team.csv");
			GenericCSVGenerator.generateListCSV(categoryTeamDTOs,
					CSVUtil.validPath(context.getProperty("path.save")) + "categories_by_team.csv",
					CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
			logger.info("File: categories_by_team.csv saved successfully");
			logger.info("File saved in path: " + CSVUtil.validPath(context.getProperty("path.save") + "categories_by_team.csv"));
			logger.debug("========================================================");
			logger.info("Exporting categories_by_team.csv completed successfully.");
			logger.debug("========================================================");

		} else {
			logger.info("Generating file categories_team.csv");
			GenericCSVGenerator.generateListCSV(categoryTeamDTOs,
					CSVUtil.validPath(context.getProperty("path.save")) + "categories_by_team_" + categoryTeamDTOs.get(0).getTeamName().replace(" ", "_") +".csv",
					CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
			logger.info("File: categories_by_team_" + categoryTeamDTOs.get(0).getTeamName() +".csv saved successfully");
			logger.info("File saved in path: " + CSVUtil.validPath(context.getProperty("path.save") + "categories_by_team_"+ categoryTeamDTOs.get(0).getTeamName().replace(" ", "_") +".csv"));
			logger.debug("========================================================");
			logger.info("Exporting categories_by_team_"+ categoryTeamDTOs.get(0).getTeamName().replace(" ", "_") +".csv completed successfully.");
			logger.debug("========================================================");
		}
	}
}
