package com.fsense.service.impl.csv;

import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.entities.Event;
import com.fsense.service.impl.writer.AuthServiceWriter;
import com.fsense.service.impl.writer.EventServiceWriter;
import com.fsense.utils.CSVUtil;
import com.fsense.utils.GenericCSVGenerator;
/***
 *  Classe que implementa escrita/leitura em CSV para serviço de Eventos
 * @author 44944_wester
 *
 */
public class EventServiceCSVWriter extends AuthServiceWriter implements EventServiceWriter {
	private static final Logger logger = Logger.getLogger(EventServiceCSVWriter.class);
	
	public EventServiceCSVWriter(Properties context) {
		super(context);
	}
	
	/***
	 * Salva os eventos em um arquivo CSV separados por dia
	 * 
	 * Este método gera um arquivo CSV contendo os eventos separadas por dia.
	 * O arquivo é salvo no caminho configurado no arquivo fsense.properties
	 * src/main/resources/export
	 * 
	 * Se nao houver eventos para a data, nenhum arquivo é gerado.
	 */
	@Override
	public void saveEvents(Properties context, List<Event> events, String date) throws Exception {
		if (events.size() > 0) {
			logger.debug("========================================================");
			logger.info("EXPORTING TO CSV");
			logger.debug("========================================================");
			logger.info("Generating file events_" + date + ".csv");
			GenericCSVGenerator.generateListCSV(events,
					CSVUtil.validPath(context.getProperty("path.save")) + "events_" + date + ".csv",
					CSVUtil.getDelimiter(context), CSVUtil.getQualifier(context));
			logger.info("File: events_" + date + ".csv successfully saved");
			logger.info("File saved in path: " + CSVUtil.validPath(context.getProperty("path.save")));
			logger.info("Export events of day " + date + " completed successfully.");
			logger.debug("========================================================");
		} else {
			logger.info("no data found for this date: " + date);
			logger.debug("========================================================");
			return;
		}
	}
}
