package com.fsense.service.impl.csv;

import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.entities.LoginHistory;
import com.fsense.service.impl.writer.AuthServiceWriter;
import com.fsense.service.impl.writer.LoginHistoryServiceWriter;
import com.fsense.utils.CSVUtil;
import com.fsense.utils.GenericCSVGenerator;
/***
 *  Classe que implementa escrita/leitura em CSV para serviço de Historico de Login
 * @author 44944_wester
 *
 */
public class LoginHistoryServiceCSVWriter extends AuthServiceWriter implements LoginHistoryServiceWriter {
	private static final Logger logger = Logger.getLogger(LoginHistoryServiceCSVWriter.class);
	
	public LoginHistoryServiceCSVWriter(Properties context) {
		super(context);
	}
	
	/***
	 * Salva os historico de login em um arquivo CSV.
	 * 
	 * Este método gera um arquivo CSV contendo os historicos de login da lista. O arquivo é
	 * salvo no caminho configurado no arquivo fsense.properties
	 * src/main/resources/export
	 */
	@Override
	public void saveLoginHistory(Properties context, List<LoginHistory> loginHistoryList) throws Exception {
		logger.debug("===============================================");
		logger.info("EXPORTING TO CSV");
		logger.debug("===============================================");
		logger.info("Generating file login_history.csv");
		GenericCSVGenerator.generateListCSV(loginHistoryList,
				CSVUtil.validPath(context.getProperty("path.save")) + "login_history.csv", CSVUtil.getDelimiter(context),
				CSVUtil.getQualifier(context));
		logger.info("File: login_history.csv saved successfully");
		logger.info("File Saved in path: " + CSVUtil.validPath(context.getProperty("path.save")));
		logger.info("Exporting login history completed successfully");
		logger.debug("===============================================");
	}
}
