package com.fsense.service.impl.database;

import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.entities.Activity;
import com.fsense.jdbc.dao.ActivityDAO;
import com.fsense.jdbc.dao.DAOFactory;
import com.fsense.service.impl.writer.ActivityServiceWriter;
import com.fsense.service.impl.writer.AuthServiceWriter;
/***
 * Classe que implementa escrita em banco de dados para serviço de Atividades
 * @author 44944_wester
 *
 */
public class ActivityServiceDatabaseWriter extends AuthServiceWriter implements ActivityServiceWriter {
	private static final Logger logger = Logger.getLogger(ActivityServiceDatabaseWriter.class);
	
	public ActivityServiceDatabaseWriter(Properties context) {
		super(context);
	}
	
	/***
	 * Salva uma lista de atividades no banco de dados
	 * 
	 * Este método salva as atividades fornecidos na lista no banco de dados 
	 * usando o ActivityDAO.Cada atividade da lista é inserido no banco de dados individualmente.
	 */
	@Override
	public void saveActivities(Properties context, List<Activity> activities, String date) throws Exception {
		ActivityDAO activityDAO = DAOFactory.createActivityDAO(context);
		int updateCount = 0;
		logger.debug("==========================================");
		logger.info("SAVING ON DATABASE");
		logger.debug("==========================================");
		logger.info("Open database Connection");
		logger.info("persisting on tb_activity");
		logger.debug("==========================================");
		
		if (activities.size() > 0) {
			activities.forEach(act -> act.setDate(date));
			for (Activity act : activities) {
				updateCount++;
				activityDAO.insert(act);
			}
		} else {
			logger.debug("No data found for this date: " + date);
			logger.debug("==========================================");
			return;
		}
	logger.info("Update Count: " + updateCount);
	logger.info("Database connection close");
	logger.debug("==========================================");	}

}
