package com.fsense.service.impl.database;

import com.fsense.entities.FSCalendar;
import com.fsense.entities.FSCalendarAssociation;
import com.fsense.jdbc.dao.DAOFactory;
import com.fsense.jdbc.dao.FSCalendarDAO;
import com.fsense.service.impl.writer.AuthServiceWriter;
import com.fsense.service.impl.writer.CalendarServiceWriter;
import org.apache.log4j.Logger;
import java.util.List;
import java.util.Properties;

public class CalendarServiceDatabaseWriter extends AuthServiceWriter implements CalendarServiceWriter {
    private static final Logger logger = Logger.getLogger(CalendarServiceDatabaseWriter.class);

    public CalendarServiceDatabaseWriter(Properties context) {
        super(context);
    }

    @Override
    public void saveCalendars(Properties context, List<FSCalendar> calendars) throws Exception {
        logger.info("Open database connection");
        logger.info("Saving on tb_calendars");
        logger.info("==========================================");

        FSCalendarDAO calendarDAO = DAOFactory.createCalendarDAO(context);
        int updateCount = 0;

        for (FSCalendar calendar : calendars) {
            updateCount++;
            calendarDAO.insert(calendar);
        }

        logger.info("Update Count: " + updateCount);
        logger.info("Database connection close");
        logger.info("==========================================");

    }

    @Override
    public void importCalendar(Properties context) throws Exception {
        logger.error("Method not available for Database Implementation");
        logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
        logger.debug("==========================================");
    }

    @Override
    public void updateCalendar(Properties context) throws Exception {
        logger.error("Method not available for Database Implementation");
        logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
        logger.debug("==========================================");
    }

    @Override
    public void saveCalendarByExternalCode(Properties context, FSCalendar calendar) throws Exception {
        logger.error("Method not available for Database Implementation");
        logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
        logger.debug("==========================================");
    }

    @Override
    public void saveCalendarById(Properties context, FSCalendar calendar) throws Exception {
        logger.error("Method not available for Database Implementation");
        logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
        logger.debug("==========================================");
    }

    @Override
    public void deleteCalendarById(Properties context) throws Exception {
        logger.error("Method not available for Database Implementation");
        logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
        logger.debug("==========================================");
    }

	@Override
	public void saveCalendarAssociations(Properties context, Integer calendarId, FSCalendarAssociation calendarAssociation) throws Exception {
		logger.error("Method not available for Database Implementation");
        logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
        logger.debug("==========================================");	
	}
}
