package com.fsense.service.impl.database;

import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.entities.Category;
import com.fsense.entities.CategoryTeam;
import com.fsense.jdbc.dao.CategoriesByTeamDAO;
import com.fsense.jdbc.dao.CategoryDAO;
import com.fsense.jdbc.dao.DAOFactory;
import com.fsense.jdbc.dao.GeneralCategoryDAO;
import com.fsense.service.impl.writer.AuthServiceWriter;
import com.fsense.service.impl.writer.CategoryServiceWriter;
/***
 * Classe que implementa escrita em banco de dados para serviço de Categorias
 * @author 44944_wester
 *
 */
public class CategoryServiceDatabaseWriter extends AuthServiceWriter implements CategoryServiceWriter {
	private static final Logger logger = Logger.getLogger(CategoryServiceDatabaseWriter.class);
	
	public CategoryServiceDatabaseWriter(Properties context) {
		super(context);
	}
	
	/***
	 * Salva uma lista de categorias no banco de dados
	 * 
	 * Este método salva as categorias fornecidos na lista no banco de dados 
	 * usando o CategoryDAO. Cada categoria da lista é inserido no banco de dados individualmente.
	 */
	@Override
	public void saveCategories(Properties context, List<Category> categories) throws Exception {
		logger.info("Open database Connection");
		logger.info("Saving on tb_categories");
		logger.debug("==========================================");

		CategoryDAO categoryDAO = DAOFactory.createCategoryDAO(context);
		int updateCount = 0;

		for (Category cat : categories) {
			updateCount++;
			cat.setUsername(context.getProperty("category.username"));
			categoryDAO.insert(cat);
		}
		logger.info("UpdateCount: " + updateCount);
		logger.info("Database connection close");
		logger.debug("==========================================");
	}
	
	/***
	 * Salva uma lista de categorias gerias no banco de dados
	 * 
	 * Este método salva as categorias gerais fornecidos na lista no banco de dados 
	 * usando o GeneralCategoryDAO. Cada categoria geral da lista é inserido no banco de dados individualmente.
	 */
	@Override
	public void saveGeneralCategories(Properties context, List<Category> categories) throws Exception {
		logger.info("Open database Connection");
		logger.info("Saving on tb_general_categories");
		logger.debug("==========================================");
		
		GeneralCategoryDAO generalCategoryDAO = DAOFactory.createGeneralCategoryDAO(context);
		int updateCount = 0;
		
		for (Category cat : categories) {
			updateCount++;
			generalCategoryDAO.insert(cat);
		}
		logger.info("UpdateCount: " + updateCount);
		logger.info("Database connection close");
		logger.debug("==========================================");
	}
	
	
	/***
	 * Salva uma lista de categorias por equipe no banco de dados
	 * 
	 * Este método salva as categorias por equipe fornecidos na lista no banco de dados 
	 * usando o CategoryByTeamDAO. Cada categoria da lista é inserido no banco de dados individualmente.
	 */
	@Override
	public void saveCategoriesByTeam(Properties context, List<CategoryTeam> categories) throws Exception {
		CategoriesByTeamDAO categoriesByTeamDAO = DAOFactory.createCategoriesByTeamDAO(context);
		int updateCount = 0;
		
		logger.info("Open database Connection");
		logger.info("Saving on tb_categories_by_team");
		logger.debug("==========================================");
		
		for(CategoryTeam cat : categories) {
			updateCount++;
			categoriesByTeamDAO.insert(cat);
		}
		logger.info("UpdateCount: " + updateCount);
		logger.info("Database connection close");
		logger.debug("==========================================");
	}
}
