package com.fsense.service.impl.database;

import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.entities.Event;
import com.fsense.jdbc.dao.DAOFactory;
import com.fsense.jdbc.dao.EventDAO;
import com.fsense.service.impl.writer.AuthServiceWriter;
import com.fsense.service.impl.writer.EventServiceWriter;
/***
 * Classe que implementa escrita em banco de dados para serviço de Eventos
 * @author 44944_wester
 *
 */
public class EventServiceDatabaseWriter extends AuthServiceWriter implements EventServiceWriter {
	private static final Logger logger = Logger.getLogger(EventServiceDatabaseWriter.class);
	
	public EventServiceDatabaseWriter(Properties context) {
		super(context);
	}
		
	/***
	 * Salva uma lista de eventos no banco de dados
	 * 
	 * Este método salva os eventos fornecidos na lista no banco de dados 
	 * usando o EventDAO. Cada evento da lista é inserido no banco de dados individualmente.
	 */
	@Override
	public void saveEvents(Properties context, List<Event> events, String date) throws Exception {
		EventDAO eventDAO = DAOFactory.createEventDAO(context);
		int updateCount = 0;
		logger.debug("==========================================");
		logger.info("SAVING ON DATABASE");
		logger.debug("==========================================");
		logger.info("Open database Connection");
		logger.info("Saving on tb_event");
		logger.debug("==========================================");
		if (events.size() > 0) {
			for (Event evt : events) {
				updateCount++;
				eventDAO.insert(evt);
			}
		} else {
			logger.info("no data found for this date: " + date);
			logger.debug("==========================================");
			return;
		}
		logger.info("Update Count: " + updateCount);
		logger.info("Database connection close");
		logger.debug("==========================================");
	}
}
