package com.fsense.service.impl.database;

import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.entities.Holiday;
import com.fsense.jdbc.dao.DAOFactory;
import com.fsense.jdbc.dao.HolidayDAO;
import com.fsense.service.impl.writer.AuthServiceWriter;
import com.fsense.service.impl.writer.HolidayServiceWriter;
/***
 * Classe que implementa escrita em banco de dados para serviço de Feriados
 * @author 44944_wester
 *
 */
public class HolidayServiceDatabaseWriter extends AuthServiceWriter implements HolidayServiceWriter {
	private static final Logger logger = Logger.getLogger(HolidayServiceDatabaseWriter.class);
	
	public HolidayServiceDatabaseWriter(Properties context) {
		super(context);
	}
	
	/***
	 * Salva uma lista de feriados no banco de dados
	 * 
	 * Este método salva os feriados fornecidos na lista no banco de dados 
	 * usando o HolidayDAO. Cada feriado da lista é inserido no banco de dados individualmente.
	 */
	@Override
	public void save(Properties context, List<Holiday> holidayList) throws Exception {
		HolidayDAO holidayDAO = DAOFactory.createHolidayDAO(context);
		int UpdateCount = 0;
		logger.debug("==========================================");
		logger.info("SAVING ON DATABASE");
		logger.debug("==========================================");
		logger.info("Open database Connection");
		logger.info("persist on tb_holiday");
		logger.debug("==========================================");
		for (Holiday holiday : holidayList) {
			UpdateCount++;
			holidayDAO.insert(holiday);
		}
		logger.info("UpdateCount: " + UpdateCount);
		logger.info("Database connection close");
		logger.debug("==========================================");
	}

	@Override
	public void importHolidays(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}

	@Override
	public void saveHolidayById(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}

	@Override
	public void updateHolidayById(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}

	@Override
	public void deleteHolidayById(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}

}
