package com.fsense.service.impl.database;

import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.entities.Person;
import com.fsense.jdbc.dao.DAOFactory;
import com.fsense.jdbc.dao.PersonDAO;
import com.fsense.service.impl.writer.AuthServiceWriter;
import com.fsense.service.impl.writer.PersonServiceWriter;
/***
 * Classe que implementa escrita em banco de dados para serviço de Pessoas
 * @author 44944_wester
 *
 */
public class PersonServiceDatabaseWriter extends AuthServiceWriter implements PersonServiceWriter {
	private static final Logger logger = Logger.getLogger(PersonServiceDatabaseWriter.class);
	
	public PersonServiceDatabaseWriter(Properties context) {
		super(context);
	}
	
	/***
	 * Salva uma lista de pessoas no banco de dados
	 * 
	 * Este método salva as pessoas fornecidos na lista no banco de dados 
	 * usando o PersonDAO. Cada pessoa da lista é inserido no banco de dados individualmente.
	 */
	@Override
	public void savePersons(Properties context, List<Person> persons) throws Exception {
		PersonDAO personDAO = DAOFactory.createPersonDAO(context);
		int updateCount = 0;
		logger.info("Open database Connection");
		logger.info("Saving on tb_person");
		logger.debug("==========================================");
		
		for(Person person : persons) {
			updateCount++;
			personDAO.insert(person);
		}
		logger.info("UpdateCount: " + updateCount);
		logger.info("Database connection close");
		logger.debug("==========================================");
	}

	@Override
	public void importPersons(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}

	@Override
	public void updatePersons(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}

	@Override
	public void exportLeaderPerson(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}

}
