package com.fsense.service.impl.database;

import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.entities.Team;
import com.fsense.jdbc.dao.DAOFactory;
import com.fsense.jdbc.dao.TeamDAO;
import com.fsense.service.impl.writer.AuthServiceWriter;
import com.fsense.service.impl.writer.TeamServiceWriter;
/***
 * Classe que implementa escrita em banco de dados para serviço de Equipes
 * @author 44944_wester
 *
 */
public class TeamServiceDatabaseWriter extends AuthServiceWriter implements TeamServiceWriter {
	private static final Logger logger = Logger.getLogger(TeamServiceDatabaseWriter.class);
	
	public TeamServiceDatabaseWriter(Properties context) {
		super(context);
	}
	
	/***
	 * Salva uma lista de equipes no banco de dados
	 * 
	 * Este método salva as equipes fornecidos na lista no banco de dados 
	 * usando o TeamDAO. Cada equipe da lista é inserido no banco de dados individualmente.
	 */
	@Override
	public void saveTeams(Properties context, List<Team> teams) throws Exception {
		TeamDAO teamDAO = DAOFactory.createTeamDAO(context);
		int updateCount = 0;
		logger.info("Open database Connection");
		logger.info("Saving on tb_team");
		logger.debug("==========================================");
		
		for(Team team : teams) {
			updateCount++;
			teamDAO.insert(team);
		}
		logger.info("UpdateCount: " + updateCount);
		logger.info("Database connection close");
		logger.debug("==========================================");
	}

	@Override
	public void importTeam(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}

	@Override
	public void updateTeamById(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}

	@Override
	public void deleteTeamById(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}

	@Override
	public void exportTeamDetailsById(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}

	@Override
	public void addMembership(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}

	@Override
	public void removeMembership(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}

	@Override
	public void addLeader(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}

	@Override
	public void removeLeader(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}
}
