package com.fsense.service.impl.database;

import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.entities.WorkdayOverride;
import com.fsense.jdbc.dao.DAOFactory;
import com.fsense.jdbc.dao.WorkdayOverrideDAO;
import com.fsense.service.impl.writer.AuthServiceWriter;
import com.fsense.service.impl.writer.WorkdayOverrideServiceWriter;
/***
 * Classe que implementa escrita em banco de dados para serviço de Trocas de Jornada
 * @author 44944_wester
 *
 */
public class WorkdayOverrideServiceDatabaseWriter extends AuthServiceWriter implements WorkdayOverrideServiceWriter {
	private static final Logger logger = Logger.getLogger(WorkdayServiceDatabaseWriter.class);
	
	public WorkdayOverrideServiceDatabaseWriter(Properties context) {
		super(context);
	}
	
	/***
	 * Salva uma lista de trocas de jornada no banco de dados
	 * 
	 * Este método salva as trocas de jornada fornecidos na lista no banco de dados 
	 * usando o WorkdayOverrideDAO. Cada troca de jornada da lista é inserido no banco de dados individualmente.
	 */
	@Override
	public void saveWorkdayOverrides(Properties context, List<WorkdayOverride> workdayOverrides) throws Exception {
		WorkdayOverrideDAO workdayOverrideDAO = DAOFactory.createWorkdayOverrideDAO(context);
		int updateCount = 0;
		logger.info("Open database Connection");
		logger.info("Saving on tb_workday_override");
		logger.debug("==========================================");
		for(WorkdayOverride workdayOverride : workdayOverrides) {
			updateCount++;
			workdayOverrideDAO.insert(workdayOverride);
		}
		logger.info("Update Count " + updateCount);
		logger.info("Database connection Close");
		logger.debug("==========================================");
	}

	@Override
	public void importWorkdayOverrides(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}

	@Override
	public void deleteWorkdayOverrideById(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}
}
