package com.fsense.service.impl.database;

import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.entities.WorkdayProrogation;
import com.fsense.jdbc.dao.DAOFactory;
import com.fsense.jdbc.dao.WorkdayProrogationDAO;
import com.fsense.service.impl.writer.AuthServiceWriter;
import com.fsense.service.impl.writer.WorkdayProrogationServiceWriter;
/***
 * Classe que implementa escrita em banco de dados para serviço de Acréscimos de Jornada
 * @author 44944_wester
 *
 */
public class
WorkdayProrogationServiceDatabaseWriter extends AuthServiceWriter implements WorkdayProrogationServiceWriter {
	private static final Logger logger = Logger.getLogger(WorkdayProrogationServiceDatabaseWriter.class);
	
	public WorkdayProrogationServiceDatabaseWriter(Properties context) {
		super(context);
	}
	
	/***
	 * Salva uma lista de acrescimos de jornada no banco de dados
	 * 
	 * Este método salva as acrescimos de jornada fornecidos na lista no banco de dados 
	 * usando o WorkdayProrogationDAO. Cada acrescimo de jornada da lista é inserido no banco de dados individualmente.
	 */
	@Override
	public void saveWorkdayProrogation(Properties context, List<WorkdayProrogation> workdayProrogations)
			throws Exception {
		WorkdayProrogationDAO workdayProrogationDAO = DAOFactory.createWorkdayProrogationDAO(context);
		int updateCount = 0;
		logger.info("Open database Connection");
		logger.info("Saving on tb_workday_prorogation");
		logger.debug("==========================================");
		for (WorkdayProrogation workdayProrogation : workdayProrogations) {
			updateCount++;
			workdayProrogationDAO.insert(workdayProrogation);		
		}
		logger.info("Update count " + updateCount);
		logger.info("Database connection close");
		logger.debug("==========================================");
	}

	@Override
	public void importWorkdayProrogation(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}

	@Override
	public void deleteWorkdayProrogationById(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}

}
