package com.fsense.service.impl.database;

import java.util.List;
import java.util.Properties;

import org.apache.log4j.Logger;

import com.fsense.entities.Workday;
import com.fsense.entities.WorkdaySchedule;
import com.fsense.jdbc.dao.DAOFactory;
import com.fsense.jdbc.dao.WorkdayDAO;
import com.fsense.jdbc.dao.WorkdayScheduleDAO;
import com.fsense.service.impl.writer.AuthServiceWriter;
import com.fsense.service.impl.writer.WorkdayServiceWriter;
/***
 * Classe que implementa escrita em banco de dados para serviço de Jornada
 * @author 44944_wester
 *
 */
public class WorkdayServiceDatabaseWriter extends AuthServiceWriter implements WorkdayServiceWriter {
	private static final Logger logger = Logger.getLogger(WorkdayServiceDatabaseWriter.class);
	
	public WorkdayServiceDatabaseWriter(Properties context) {
		super(context);
	}
	
	/***
	 * Salva uma lista de jornadas no banco de dados
	 * Juntamente com os dias da semana referente a jornada
	 * 
	 * Este método salva as de jornadas fornecidos na lista no banco de dados 
	 * usando o WorkdayDAO. Cada troca jornada da lista é inserido no banco de dados individualmente.
	 * para cada jornada salva, tbm são salvos os dias da semana associados com essa jornada
	 */
	@Override
	public void saveWorkdays(Properties context, List<Workday> workdays) throws Exception {
		WorkdayDAO workdayDAO = DAOFactory.createWorkdayDAO(context);
		WorkdayScheduleDAO workdayScheduleDAO = DAOFactory.createWorkdayScheduleDAO(context);
		int updateCount = 0;
		logger.info("Open database Connection");
		logger.info("Saving on tb_workday");
		logger.debug("==========================================");
		for (Workday workday : workdays) {
			updateCount++;
			workdayDAO.insert(workday);
			for(WorkdaySchedule schedule : workday.getWorkdayScheduleList()) {
				workdayScheduleDAO.insert(schedule, workday.getId());
			}
		}
		logger.info("UpdateCount: " + updateCount);
		logger.info("Database connection close");
		logger.debug("==========================================");
	}

	@Override
	public void importWorkdays(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}

	@Override
	public void saveWorkdayById(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}

	@Override
	public void updateWorkdayById(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}

	@Override
	public void deleteWorkdayById(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}

	@Override
	public void associatePeople(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}

	@Override
	public void associateTeams(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
	}

	@Override
	public void disassociatePeople(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}

	@Override
	public void disassociateTeams(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}

	@Override
	public void exportPersonsAssociatedWithWorkdayById(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}

	@Override
	public void exportTeamsAssociatedWithWorkdayById(Properties context) throws Exception {
		logger.error("Method not available for Database Implementation");
		logger.error("To use CSV Implementation, set property 'use.database' to false in file fsense.properties");
		logger.debug("==========================================");
	}
}
