package com.fsense.utils;

import java.sql.Timestamp;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;

import org.joda.time.DateTime;
import org.joda.time.DateTimeZone;
import org.joda.time.Days;
import org.joda.time.Duration;
import org.joda.time.DurationFieldType;
import org.joda.time.LocalDate;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.joda.time.format.PeriodFormatter;
import org.joda.time.format.PeriodFormatterBuilder;

import com.fsense.exceptions.InvalidDateException;

public class DateUtils {

	public static final String DAY_MONTH_YEAR = "dd-MM-yyyy";

	public static final String DATE_FORMAT_SQL = "yyyy-MM-dd HH:mm:ss";

	public static final String DATETIME_FORMAT = "yyyy-MM-dd'T'HH:mm:ssZZ";

	public static final String TIME_WITHOUT_TIMEZONE = "HH:mm:ss";

	public static final String DEFAULT_DATE_FORMAT = "E MMM dd HH:mm:ss z yyyy";

	public static final String YEAR_MONTH_DAY = "yyyy-MM-dd";

	private static SimpleDateFormat inputFormat = new SimpleDateFormat("dd/MM/yyyy");
	private static SimpleDateFormat outputFormat = new SimpleDateFormat(DATETIME_FORMAT);

	public static List<Date> daysBetweenAndIncluding(LocalDate initialDate, LocalDate finalDate) {
		int days = Days.daysBetween(initialDate, finalDate).getDays();
		List<Date> dates = new ArrayList<Date>(days);
		for (int i = 0; i <= days; i++) {
			LocalDate d = initialDate.withFieldAdded(DurationFieldType.days(), i);
			dates.add(d.toDate());
		}
		return dates;
	}
	
	public static DateTime convertDateToDateTime(Date date) {
		return new DateTime(date);
	}
	
	public static DateTime apply00h00m00s(DateTime datetime) {
		return datetime.withTimeAtStartOfDay();
	}
	
	public static DateTime apply23h59m59s(DateTime datetime) {
		return datetime.withTime(23, 59, 59, 999);
	}
	
	public static DateTime applyCustomTime(DateTime datetime, String customTime) {
		String [] parts = customTime.split(":");
		int hours = Integer.parseInt(parts[0]);
		int minutes = Integer.parseInt(parts[1]);
		int seconds = Integer.parseInt(parts[2]);
		return datetime.withTime(hours, minutes, seconds, 0);
 	}

	public static Date aplicar0H0m0s(Date date) {
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		cal.set(Calendar.HOUR_OF_DAY, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		return cal.getTime();
	}

	public static Date aplicar23H59m59s(Date date) {
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		cal.set(Calendar.HOUR_OF_DAY, 23);
		cal.set(Calendar.MINUTE, 59);
		cal.set(Calendar.SECOND, 59);
		return cal.getTime();
	}

	public static String apply00H00m00s(String date) throws ParseException {
		Date inputDate = inputFormat.parse(date);
		Calendar cal = Calendar.getInstance();
		cal.setTime(inputDate);
		cal.set(Calendar.HOUR_OF_DAY, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		String outputDate = outputFormat.format(cal.getTime());
		return outputDate;
	}

	public static String apply23H59m59s(String date) throws ParseException {
		Date inputDate = inputFormat.parse(date);
		Calendar cal = Calendar.getInstance();
		cal.setTime(inputDate);
		cal.set(Calendar.HOUR_OF_DAY, 23);
		cal.set(Calendar.MINUTE, 59);
		cal.set(Calendar.SECOND, 59);
		String outputDate = outputFormat.format(cal.getTime());
		return outputDate;
	}

	public static String applyIntialHour(String date, String initalHour) throws ParseException {
		Date inputDate = inputFormat.parse(date);
		Calendar cal = Calendar.getInstance();
		cal.setTime(inputDate);
		String[] parts = initalHour.split(":");
		int hours = Integer.parseInt(parts[0]);
		int minutes = Integer.parseInt(parts[1]);
		int seconds = Integer.parseInt(parts[2]);
		cal.set(Calendar.HOUR_OF_DAY, hours);
		cal.set(Calendar.MINUTE, minutes);
		cal.set(Calendar.SECOND, seconds);
		String outputDate = outputFormat.format(cal.getTime());
		return outputDate;
	}

	public static String applyFinalHour(String date, String finalHour) throws ParseException {
		Date inputDate = inputFormat.parse(date);
		Calendar cal = Calendar.getInstance();
		cal.setTime(inputDate);
		String[] parts = finalHour.split(":");
		int hours = Integer.parseInt(parts[0]);
		int minutes = Integer.parseInt(parts[1]);
		int seconds = Integer.parseInt(parts[2]);
		cal.set(Calendar.HOUR_OF_DAY, hours);
		cal.set(Calendar.MINUTE, minutes);
		cal.set(Calendar.SECOND, seconds);
		String outputDate = outputFormat.format(cal.getTime());
		return outputDate;
	}

	public static Date addTime(Date date, String time) throws ParseException {

		SimpleDateFormat dtf = new SimpleDateFormat(TIME_WITHOUT_TIMEZONE);
		Date dt = dtf.parse(time);
		Calendar ctime = Calendar.getInstance();
		ctime.setTime(dt);
		Calendar cal = Calendar.getInstance();
		cal.setTime(date);
		cal.set(Calendar.HOUR_OF_DAY, ctime.get(Calendar.HOUR_OF_DAY));
		cal.set(Calendar.MINUTE, ctime.get(Calendar.MINUTE));
		cal.set(Calendar.SECOND, ctime.get(Calendar.SECOND));
		return cal.getTime();
	}

	public static String getPrettyFormat(long milliseconds) {
		Duration duration = new Duration(milliseconds);
		PeriodFormatter formatter = new PeriodFormatterBuilder().appendDays().appendSuffix("d").appendHours()
				.appendSuffix("h").appendMinutes().appendSuffix("m").appendSeconds().appendSuffix("s").toFormatter();
		return formatter.print(duration.toPeriod());
	}

	public static List<Date> daysBetweenAndIncluding(Date initialDate, Date finalDate) {
		LocalDate initialDateFields = LocalDate.fromDateFields(DateUtils.aplicar0H0m0s(initialDate));
		LocalDate finalDateFields = LocalDate.fromDateFields(DateUtils.aplicar23H59m59s(finalDate));
		return DateUtils.daysBetweenAndIncluding(initialDateFields, finalDateFields);
	}

	public static String formatDateTime(DateTime dateTime) {

		if (dateTime == null) {
			return null;
		}

		return format(dateTime, DATETIME_FORMAT);

	}

	public static String formatDateSQL(DateTime dateTime) {

		if (dateTime == null) {
			return null;
		}

		return format(dateTime, DATE_FORMAT_SQL);

	}

	public static String format(DateTime dateTime, String format) {

		if (dateTime == null) {
			return null;
		}

		DateTimeFormatter dtf = DateTimeFormat.forPattern(format);

		return dtf.print(dateTime);
	}

	public static String format(DateTime dateTime, String format, String timeZone) {

		if (dateTime == null) {
			return null;
		}

		dateTime = dateTime.toDateTime(DateTimeZone.forID(timeZone));

		DateTimeFormatter dtf = DateTimeFormat.forPattern(format);
		return dtf.print(dateTime);
	}

	public static DateTime parse(String date, String format) {

		if (StringUtils.isEmpty(date) || StringUtils.isEmpty(format)) {
			return null;
		}
		return DateTime.parse(date);
	}

	public static String DateFormat(String date) {
		java.time.format.DateTimeFormatter inputFormatter = java.time.format.DateTimeFormatter.ofPattern("dd/MM/yyyy");
		java.time.LocalDate localDate = java.time.LocalDate.parse(date, inputFormatter);
		LocalDateTime localDateTime = localDate.atStartOfDay();
		ZoneOffset zoneOffset = ZoneOffset.UTC;
		java.time.format.DateTimeFormatter outputFormater = java.time.format.DateTimeFormatter
				.ofPattern(DATETIME_FORMAT);
		String formattedDate = localDateTime.atOffset(zoneOffset).format(outputFormater);
		return formattedDate;
	}

	public static DateTime parseDateTime(String dateTime) {
		return parse(dateTime, DATETIME_FORMAT);
	}

	public static DateTime parseDateSQL(String dateTime) {
		return parse(dateTime, DATE_FORMAT_SQL);
	}

	public static DateTime parseDateTime(String dateTime, String format) {
		return parse(dateTime, format);
	}

	public static void validateDates(String initialDate, String finalDate) throws Exception {
		try {
			Date startDate = inputFormat.parse(initialDate);
			Date endDate = inputFormat.parse(finalDate);
			if (startDate.after(endDate)) {
				throw new InvalidDateException("A data inicial é maior que a data final");
			}
		} catch (ParseException e) {
			throw new InvalidDateException("Formato de data invalido, Use o formato dd/MM/yyyy");
		}
	}

	public static String parseData(Date inputDate, String format) {
		DateFormat outputFormat = new SimpleDateFormat(format);
		outputFormat.setTimeZone(TimeZone.getDefault());
		String outputDate = outputFormat.format(inputDate);
		return outputDate;
	}

	public static String parseDataToString(String inputDate, String format) {
		try {
			SimpleDateFormat outputFormat = new SimpleDateFormat(YEAR_MONTH_DAY);
			Date data = inputFormat.parse(inputDate);
			String outputDate = outputFormat.format(data);
			return outputDate;
		} catch (ParseException e) {
			throw new InvalidDateException("Formato de data invalido, Use o formato dd/MM/yyyy");
		}
	}

	public static String formatDateToString(Date date, String format) {
		SimpleDateFormat sdf = new SimpleDateFormat(format);
		return sdf.format(date);
	}
	
	public static Date parseStringToDate(String datestr) throws ParseException {
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		Date date = sdf.parse(datestr);
		return date;
	}
	
	public static Timestamp parseStringToTimestamp(String dateStr) {
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
		try {
			Date parsedDate = sdf.parse(dateStr);
			return new Timestamp(parsedDate.getTime());
		} catch (ParseException e) {
			e.printStackTrace();
			return null;
		}
		
	}
	
	public static List<Date> getDateInterval(String startDateStr, String endDateStr) throws ParseException {
		SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
		Date startDate = sdf.parse(startDateStr);
		Date endDate = sdf.parse(endDateStr);
		
		List<Date> dateList = new ArrayList<>();
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(startDate);
		
		while(!calendar.getTime().after(endDate)) {
			dateList.add(calendar.getTime());
			calendar.add(Calendar.DATE, 1);
		}
		return dateList;
		}
	
	public static String formatDate(Date day) {
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
		return sdf.format(day);
	}
	
	@SuppressWarnings("deprecation")
	public static String getDateAndApply00H00m00s(Date day) {
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
		day.setHours(0);
		day.setMinutes(0);
		day.setSeconds(0);
		return sdf.format(day);
	}
	
	@SuppressWarnings("deprecation")
	public static String getDateAndApply23H59m59s(Date day) {
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
		day.setHours(23);
		day.setMinutes(59);
		day.setSeconds(59);
		return sdf.format(day);
	}
	
	@SuppressWarnings("deprecation")
	public static String getDateAndApplyCustomTime(Date day, String customTime) throws ParseException {
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'z'");
		SimpleDateFormat customFormat = new SimpleDateFormat("HH:mm:ss");
		Date customDate = customFormat.parse(customTime);
		day.setHours(customDate.getHours());
		day.setMinutes(customDate.getMinutes());
		day.setSeconds(customDate.getSeconds());
		return sdf.format(day);
	}
}
