package com.fsense.utils;

 import java.io.FileReader;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVParser;
import org.apache.commons.csv.CSVRecord;

import com.opencsv.CSVReader;
import com.opencsv.CSVReaderBuilder;
 import org.apache.log4j.Logger;
/***
 * Classe responsavel por fazer leitura de arquivos do tipo CSV.
 * @author 44944_wester
 *
 */
public class GenericCSVReader {
    private static final Logger logger = Logger.getLogger(GenericCSVReader.class);
    
	public static <T> List<T> readCSV(String csvFile, Class<T> objectType, char delimiter, char quotePair) {
        List<T> objects = new ArrayList<>();

        try (FileReader fileReader = new FileReader(csvFile);
             CSVParser csvParser = new CSVParser(fileReader, CSVFormat.DEFAULT.withDelimiter(delimiter).withQuote(quotePair).withHeader())) {

            Constructor<T> constructor = objectType.getDeclaredConstructor();

            for (CSVRecord csvRecord : csvParser) {
                T object = constructor.newInstance();

                Field[] fields = objectType.getDeclaredFields();
                for (Field field : fields) {
                    String columnName = field.getName();
                    String fieldValue = csvRecord.get(columnName);

                    field.setAccessible(true);
                    Object convertedValue = null;
                    if(!fieldValue.isEmpty()) {
                    	convertedValue = convertValue(fieldValue, field.getType());
                    }
                    field.set(object, convertedValue);
                }

                objects.add(object);
            }
        } catch (IOException | ReflectiveOperationException e) {
            logger.error("error occurred when trying to read the file, please check if the file path is correct or informed on fsense.properties");
        }

        return objects;
    }
	
	public static <T> List<T> readCSVWithoutMapping(String path, Class<T> clazz, char delimiter, char quotePair) {
        List<T> dataList = new ArrayList<>();

        try (CSVReader reader = new CSVReaderBuilder(new FileReader(path))
                .withCSVParser(new com.opencsv.CSVParserBuilder().withSeparator(delimiter).withQuoteChar(quotePair).build())
                .build()) {

            String[] nextLine;
            while ((nextLine = reader.readNext()) != null) {
                for (String value : nextLine) {
                    @SuppressWarnings("unchecked")
					T convertedValue = (T) convertValue(value, clazz);
                    dataList.add(convertedValue);
                }
            }
        } catch (Exception e) {
            logger.error("error occurred when trying to read the file, please check if the file path is correct");
        }

        return dataList;
    }

    private static Object convertValue(String value, Class<?> targetType) {
        if (targetType == String.class) {
            return value;
        } else if (targetType == int.class || targetType == Integer.class) {
            return Integer.parseInt(value);
        } else if (targetType == long.class || targetType == Long.class) {
            return Long.parseLong(value);
        } else if (targetType == double.class || targetType == Double.class) {
            return Double.parseDouble(value);
        } else if (targetType == boolean.class || targetType == Boolean.class) {
            return Boolean.parseBoolean(value);
        }
        return null;
    }
}