package com.fsense.utils;

import java.util.Iterator;
import java.util.Properties;

public class StringUtils {
	
	/**
	 * Indica se é null ou uma String 'vazia'
	 * 
	 * @param c
	 * @return
	 */
	public static boolean isEmpty(String value) {
		return value == null || value.trim().equals("");
	}

	/**
	 * Se valor informado é nulo, retorna o literal <code>""</code>. Caso contrário, retorna o próprio valor informado.
	 * 
	 * @param value
	 */
	public static String nonEmptyIfNull(String value) {
		return value == null ? "" : value;
	}

	/**
	 * Compara duas Strings com equals evitando nullPointer
	 */
	public static boolean equalsNullable(String s1, String s2) {
		if (s1 == null && s2 == null) {
			return true;
		}
		if (s1 != null && s2 != null) {
			return s1.equalsIgnoreCase(s2);
		}
		return false;
	}

	public static String join(final Iterator<?> iterator, final String separator) {
        // handle null, zero and one elements before building a buffer
        if (iterator == null) {
            return null;
        }
        if (!iterator.hasNext()) {
            return "";
        }
        final Object first = iterator.next();
        if (!iterator.hasNext()) {
            final String result = first.toString();
            return result;
        }

        // two or more elements
        final StringBuilder buf = new StringBuilder(256);
        if (first != null) {
            buf.append(first);
        }

        while (iterator.hasNext()) {
            if (separator != null) {
                buf.append(separator);
            }
            final Object obj = iterator.next();
            if (obj != null) {
                buf.append(obj);
            }
        }
        return buf.toString();
    }

	public static boolean isBlank(String str) {
		return str == null || str.isEmpty();
	}
	
	public static String getPath(Properties context, String property) {
		String path = context.getProperty(property);
		if(!path.endsWith("/")) {
			path = path + "/";
			return path;
		}
		return path;
	}

}
