CREATE DATABASE db_fsense;

use db_fsense;

CREATE TABLE tb_team (
	id BIGINT PRIMARY KEY,
    name VARCHAR(255) NOT NULL,
    workday_id INT,
    team_parent_id BIGINT,
    team_parent_name VARCHAR(255),
    FOREIGN KEY (team_parent_id) REFERENCES tb_team(id)
);

CREATE INDEX idx_team_parent_id ON tb_team(team_parent_id);

CREATE TABLE tb_person (
	id BIGINT PRIMARY KEY,
    name VARCHAR(255),
    email VARCHAR(255),
    device_identifier VARCHAR(255),
    team_id BIGINT,
    team_name VARCHAR(255),
    workday_id INT,
    can_delete BOOLEAN,
    root BOOLEAN,
    calendar_id INT
);


CREATE TABLE tb_category (
    id BIGINT PRIMARY KEY auto_increment, 
    application_name VARCHAR(255), 
    category CHAR(1),
	category_name VARCHAR(255),
    username VARCHAR(255)
);

ALTER table tb_category
ADD constraint unique_category_constraint
UNIQUE (application_name, category, category_name);


CREATE TABLE tb_general_category (
    id BIGINT PRIMARY KEY AUTO_INCREMENT, 
    application_name VARCHAR(255), 
    category CHAR(1),
	category_name VARCHAR(255),
    username VARCHAR(255)
);

ALTER TABLE tb_general_category
ADD constraint unique_general_category_constraint
UNIQUE (application_name, category, category_name);

CREATE TABLE tb_categories_by_team (
	id BIGINT PRIMARY KEY AUTO_INCREMENT,
    application_name VARCHAR(255), 
	category CHAR(1),
	category_name VARCHAR(255),
    team_id BIGINT,
    team_name  VARCHAR(255)
);

ALTER TABLE tb_categories_by_team
ADD constraint unique_categories_by_team_constraint
UNIQUE (application_name, team_id, team_name);

CREATE TABLE tb_activity (
	id BIGINT PRIMARY KEY auto_increment,
    date DATE,
    application VARCHAR(255), 
    login VARCHAR(255), 
    process_name VARCHAR(255), 
    evt_duration_milliseconds BIGINT,
    host VARCHAR(255), 
    category CHAR(1), 
    category_name VARCHAR(255),
    UNIQUE key unique_activity(date, login, (IFNULL(process_name, '')), evt_duration_milliseconds)
);

CREATE TABLE tb_event (
	id BIGINT PRIMARY KEY auto_increment,
    application VARCHAR(255),
    login VARCHAR(255),
    process_name VARCHAR(255),
    evt_duration_milliseconds BIGINT,
    host VARCHAR(255),
    url VARCHAR(1024),
    machine VARCHAR(255),
    internal_ip VARCHAR(255),
    evt_fired TIMESTAMP,
    evt_stopped TIMESTAMP,
    creation_date TIMESTAMP,
    mouse_clicks_per_millisecond DOUBLE,
    keystrokes_per_millisecond DOUBLE,
    keystrokes_count INT,
    mouse_clicks_count INT,
    UNIQUE KEY unique_event(application, login, (IFNULL(process_name, '')), evt_duration_milliseconds, evt_fired)
);

CREATE TABLE tb_workday (
	id INT PRIMARY KEY,
    name VARCHAR(255),
    work_national_holidays BOOLEAN,
    work_regional_holidays BOOLEAN,
    is_flexible BOOLEAN,
    ignore_non_focus_longer_than TIME,
    calendar_id INT
);

CREATE table tb_workday_schedule (
	weekday ENUM('MONDAY', 'TUESDAY', 'WEDNESDAY', 'THURSDAY', 'FRIDAY', 'SATURDAY', 'SUNDAY'),
    start_time TIME,
    end_time TIME,
    max_flexible_duration TIME,
    workday_id INT,
    CONSTRAINT check_weekday CHECK (weekday IN ('MONDAY', 'TUESDAY', 'WEDNESDAY', 'THURSDAY', 'FRIDAY', 'SATURDAY', 'SUNDAY'))
);

ALTER TABLE tb_workday_schedule
ADD CONSTRAINT uq_weekday_start_end UNIQUE(weekday, start_time, end_time, workday_id);

ALTER TABLE tb_workday_schedule
ADD CONSTRAINT uq_weekday_max_flex_duration UNIQUE(weekday, max_flexible_duration, workday_id);


CREATE TABLE tb_workday_override (
	id bigint PRIMARY KEY,
    person_id BIGINT,
    date Date,
    start_time TIME,
    interval_start_time TIME,
    end_time TIME,
    interval_end_time time,
    creator_id BIGINT,
    message VARCHAR(255),
    max_flexible_duration TIME
);

CREATE TABLE tb_workday_prorogation (
	id BIGINT PRIMARY KEY,
    source VARCHAR(255),
    person_id BIGINT,
    request_message VARCHAR(255),
    date DATE,
    start_time TIME,
    end_time TIME,
    extra_time TIME,
    status varchar(255)
);

CREATE TABLE tb_login_history(
	id BIGINT primary key AUTO_INCREMENT,
    name VARCHAR(255),
    email VARCHAR(255),
    date TIMESTAMP
);

CREATE TABLE tb_holiday (
	id BIGINT PRIMARY KEY,
    name VARCHAR(255),
    date TIMESTAMP,
    type ENUM('NATIONAL', 'REGIONAL'),
    can_repeat boolean,
    year int,
    month int,
    day int,
    calendar_id INT
);

CREATE TABLE tb_keystrokes (
	id BIGINT PRIMARY KEY AUTO_INCREMENT, 
	machine_name VARCHAR(255),
    user_name VARCHAR(255),
    creation_date TIMESTAMP,
    person_id INT,
    device_id INT,
    keystrokes_count INT,
    mouse_clicks_count INT,
    keystrokes_per_millisecond DOUBLE,
    mouse_clicks_per_millisecond DOUBLE,
    start_time TIMESTAMP,
    end_time TIMESTAMP, 
    duration INT
);

CREATE TABLE tb_calendar (
    id BIGINT PRIMARY KEY AUTO_INCREMENT,
    name VARCHAR(255),
    external_code VARCHAR(255)
);