CREATE DATABASE db_fsense;

CREATE TABLE tb_team (
	id SERIAL PRIMARY KEY,
	name VARCHAR(255),
	workday_id INT,
	team_parent_id INT,
	team_parent_name VARCHAR(255)
);

CREATE INDEX idx_team_parent_id ON tb_team(team_parent_id);

CREATE TABLE tb_person (
	id SERIAL PRIMARY KEY,
	name VARCHAR(255),
	email VARCHAR(255),
	device_identifier VARCHAR(255),
	team_id INT,
	team_name VARCHAR(255),
	workday_id INT,
	can_delete BOOLEAN,
	root BOOLEAN,
	calendar_id INT
);

CREATE TABLE tb_category (
    id BIGSERIAL PRIMARY KEY,
    application_name VARCHAR(255),
    category CHAR(1),
    category_name VARCHAR(255),
    username VARCHAR(255)
);

ALTER TABLE tb_category
ADD CONSTRAINT unique_category_constraint
UNIQUE (application_name, category, category_name, username);


CREATE TABLE tb_general_category (
    id BIGSERIAL PRIMARY KEY,
    application_name VARCHAR(255),
    category CHAR(1),
    category_name VARCHAR(255)
);

ALTER TABLE tb_general_category
ADD CONSTRAINT unique_general_category_constraint
UNIQUE (application_name, category, category_name);

CREATE TABLE tb_categories_by_team (
	id BIGSERIAL PRIMARY KEY,
	application_name VARCHAR(255),
    category CHAR(1),
    category_name VARCHAR(255),
    team_id BIGINT,
    team_name VARCHAR(255)
);

ALTER TABLE tb_categories_by_team
ADD CONSTRAINT unique_categories_by_team_constraint
UNIQUE (application_name, team_id, team_name);

CREATE TABLE tb_activity (
    id BIGSERIAL PRIMARY KEY,
    date DATE,
    application VARCHAR(255),
    login VARCHAR(255),
    process_name VARCHAR(255),
    evt_duration_milliseconds BIGINT,
    host VARCHAR(255),
    category CHAR(1),
    category_name VARCHAR(255)
);

ALTER TABLE tb_activity
ADD CONSTRAINT unique_activity_constraint UNIQUE (date, login, evt_duration_milliseconds);

CREATE TABLE tb_event (
    id BIGSERIAL PRIMARY KEY,
    application VARCHAR(255),
    login VARCHAR(255),
    process_name VARCHAR(255),
    evt_duration_milliseconds BIGINT,
    host VARCHAR(255),
    url VARCHAR(1024),
    machine VARCHAR(255),
    internal_ip VARCHAR(255),
    evt_fired TIMESTAMP,
    evt_stopped TIMESTAMP,
    creation_date TIMESTAMP,
    mouse_clicks_per_millisecond DOUBLE PRECISION,
    keystrokes_per_millisecond DOUBLE PRECISION,
    keystrokes_count INT,
    mouse_clicks_count INT
);

ALTER TABLE tb_event
ADD CONSTRAINT unique_event_constraint UNIQUE (application, login, evt_duration_milliseconds, evt_fired);

CREATE TABLE tb_workday (
    id SERIAL PRIMARY KEY,
    name VARCHAR(255),
    work_national_holidays BOOLEAN,
    work_regional_holidays BOOLEAN,
    is_flexible BOOLEAN,
    ignore_non_focus_longer_than TIME,
    calendar_id INT
);

CREATE TABLE tb_workday_schedule (
    id SERIAL PRIMARY KEY,
    weekday text CHECK (weekday IN('MONDAY', 'TUESDAY', 'WEDNESDAY', 'THURSDAY', 'FRIDAY', 'SATURDAY', 'SUNDAY')),
    start_time TIME,
    end_time TIME,
    max_flexible_duration TIME,
    workday_id INT,

    FOREIGN KEY (workday_id) REFERENCES tb_workday(id)
);

CREATE UNIQUE INDEX uq_weekday_start_end ON tb_workday_schedule (weekday, start_time, end_time, workday_id);
CREATE UNIQUE INDEX uq_weekday_max_flexible_duration ON tb_workday_schedule (weekday, max_flexible_duration, workday_id);


CREATE TABLE tb_workday_override (
    id BIGSERIAL PRIMARY KEY,
    person_id BIGINT,
    date DATE,
    start_time TIME,
    interval_start_time TIME,
    end_time TIME,
    interval_end_time TIME,
    creator_id BIGINT,
    message VARCHAR(255),
    max_flexible_duration TIME
);


CREATE TABLE tb_workday_prorogation (
    id BIGSERIAL PRIMARY KEY,
    source VARCHAR(255),
    person_id  BIGINT,
    request_message VARCHAR(255),
    date DATE,
    start_time TIME,
    end_time TIME,
    extra_time TIME,
    status VARCHAR(255)
);

CREATE TABLE tb_login_history (
    id BIGSERIAL PRIMARY KEY,
    name VARCHAR(255),
    email VARCHAR(255),
    date TIMESTAMP
);

CREATE TABLE tb_holiday (
    id BIGSERIAL PRIMARY KEY,
    name VARCHAR(255),
    date TIMESTAMP,
    type VARCHAR(255) CHECK (type IN ('NATIONAL', 'REGIONAL')),
    can_repeat BOOLEAN,
    year INT,
    month INT,
    day INT,
    calendar_id INT
);

CREATE TABLE tb_keystrokes (
    id BIGSERIAL PRIMARY KEY,
    machine_name VARCHAR(255),
    user_name VARCHAR(255),
    creation_date TIMESTAMP,
    person_id INT,
    device_id INT,
    keystrokes_count INT,
    mouse_clicks_count INT,
    keystrokes_per_millisecond DOUBLE PRECISION,
    mouse_clicks_per_millisecond DOUBLE PRECISION,
    start_time TIMESTAMP,
    end_time TIMESTAMP,
    duration INT
);

CREATE TABLE tb_calendar (
    id BIGSERIAL PRIMARY KEY,
    name VARCHAR(255),
    external_code VARCHAR(155)
)