CREATE DATABASE db_fsense;

USE db_fsense;

CREATE TABLE tb_team (
	id INT PRIMARY KEY,
	name NVARCHAR(255) NOT NULL,
	workday_id INT,
	team_parent_id INT NULL,
	team_parent_name NVARCHAR(255) NULL
);

CREATE INDEX idx_team_parent_id ON tb_team(team_parent_id);

CREATE TABLE tb_person (
	id INT PRIMARY KEY,
	name NVARCHAR(255),
	email NVARCHAR(255),
	device_identifier VARCHAR(255),
	team_id INT,
	team_name VARCHAR(255),
	workday_id INT,
	can_delete BIT,
	root BIT,
	calendar_id INT
);


CREATE TABLE tb_category (
    id INT IDENTITY(1,1) PRIMARY KEY,
    application_name VARCHAR(255),
    category CHAR(1),
    category_name VARCHAR(255),
    username VARCHAR(255)
);

ALTER TABLE tb_category
ADD CONSTRAINT unique_category_constraint
UNIQUE (application_name, category, category_name, username);


CREATE TABLE tb_general_category (
    id INT IDENTITY(1,1) PRIMARY KEY,
    application_name VARCHAR(255),
    category CHAR(1),
    category_name VARCHAR(255)
);

ALTER TABLE tb_general_category
ADD CONSTRAINT unique_general_category_constraint
UNIQUE (application_name, category, category_name);

CREATE TABLE tb_categories_by_team (
    id INT IDENTITY(1,1) PRIMARY KEY,
    application_name VARCHAR(255),
    category CHAR(1),
    category_name VARCHAR(255),
    team_id INT,
    team_name VARCHAR(255)
);

ALTER TABLE tb_categories_by_team
ADD CONSTRAINT unique_categories_by_team_constraint
UNIQUE (application_name, team_id, team_name);

CREATE TABLE tb_activity (
    id INT IDENTITY(1,1) PRIMARY KEY,
    date DATE,
    application VARCHAR(255),
    login VARCHAR(255),
    process_name VARCHAR(255),
    evt_duration_milliseconds BIGINT,
    host VARCHAR(255),
    category CHAR(1),
    category_name VARCHAR(255)
);

ALTER TABLE tb_activity
ADD CONSTRAINT unique_activity_constraint UNIQUE (date, login, evt_duration_milliseconds);

CREATE TABLE tb_event (
    id INT IDENTITY(1,1) PRIMARY KEY,
    application VARCHAR(255),
    login VARCHAR(255),
    process_name VARCHAR(255),
    evt_duration_milliseconds BIGINT,
    host VARCHAR(255),
    url VARCHAR(1024),
    machine VARCHAR(255),
    internal_ip VARCHAR(255),
    evt_fired DATETIME,
    evt_stopped DATETIME,
    creation_date DATETIME
);

ALTER TABLE tb_event
ADD CONSTRAINT unique_event_constraint UNIQUE (application, login, evt_duration_milliseconds, evt_fired);

CREATE TABLE tb_workday (
    id INT PRIMARY KEY,
    name NVARCHAR(255),
    work_national_holidays BIT,
    work_regional_holidays BIT,
    is_flexible BIT,
    ignore_non_focus_longer_than TIME,
    calendar_id INT
);


CREATE TABLE tb_workday_schedule (
     id INT IDENTITY(1,1) PRIMARY KEY,
    weekday VARCHAR(255) CHECK (weekday IN ('MONDAY', 'TUESDAY', 'WEDNESDAY', 'THURSDAY', 'FRIDAY', 'SATURDAY', 'SUNDAY')),
    start_time TIME,
    end_time TIME,
    max_flexible_duration TIME,
    workday_id INT,

    FOREIGN KEY (workday_id) REFERENCES tb_workday(id)
);

ALTER TABLE tb_workday_schedule
ADD CONSTRAINT uq_weekday_start_end UNIQUE (weekday, start_time, end_time, max_flexible_duration, workday_id);

CREATE TABLE tb_workday_override (
    id BIGINT PRIMARY KEY,
    person_id BIGINT,
    date DATE,
    start_time TIME,
    interval_start_time TIME,
    end_time TIME,
    interval_end_time TIME,
    creator_id BIGINT,
    message NVARCHAR(255),
    max_flexible_duration TIME
);

CREATE TABLE tb_workday_prorogation (
    id BIGINT PRIMARY KEY,
    source NVARCHAR(255),
    person_id BIGINT,
    request_message NVARCHAR(255),
    date DATE,
    start_time TIME,
    end_time TIME,
    extra_time TIME,
    status NVARCHAR(255)
);

CREATE TABLE tb_login_history (
    id BIGINT IDENTITY(1,1) PRIMARY KEY,
    name NVARCHAR(255),
    email NVARCHAR(255),
    date DATETIME
);

CREATE TABLE tb_holiday (
    id BIGINT PRIMARY KEY,
    name NVARCHAR(255),
    date DATETIME,
    type NVARCHAR(255) CHECK (type IN ('NATIONAL', 'REGIONAL')),
    can_repeat BIT,
    year INT,
    month INT,
    day INT,
    calendar_id INT
);

CREATE TABLE tb_keystrokes (
    id BIGINT IDENTITY(1,1) PRIMARY KEY,
    machine_name NVARCHAR(255),
    user_name NVARCHAR(255),
    creation_date DATETIME,
    person_id INT,
    device_id INT,
    keystrokes_count INT,
    mouse_clicks_count INT,
    keystrokes_per_millisecond FLOAT,
    mouse_clicks_per_millisecond FLOAT,
    start_time DATETIME,
    end_time DATETIME,
    duration INT
);

CREATE TABLE tb_calendar (
    id BIGINT PRIMARY KEY,
    name NVARCHAR(255),
    external_code NVARCHAR(255)
)